from LaborDtl import LaborDtl
from NoteByLaborDtlFactory import NoteByLaborDtlFactory
from Store import store
from sndcs_common.DateTimeMixin import DateTimeMixin
import datetime

class LaborDtlFactory(DateTimeMixin):

    def new(self, operation = None, indirect = None, jobType = "Production", startStamp = None, startTerminal = None, notes = None):
        assert operation or indirect
        assert not (operation and indirect)
        dtl = LaborDtl()
        if operation:
            dtl.setOperation(operation)
            dtl.setJobType(jobType)
        if indirect:
            dtl.setIndirect(indirect)
            dtl.setJobType("Indirect")
        if not startStamp:
            startStamp = self.now()
        dtl.setStartStamp(startStamp)
        if startTerminal:
            dtl.setStartTerminal(startTerminal)
        if notes:
            note = NoteByLaborDtlFactory().new(notes)
            dtl.addToNotes(note)
        dtl.setActiveTrans(1)
        
        return dtl

    def getLaborDtlBySerialNum(self, serialNum):
        return store.fetchObject(LaborDtl, serialNum)

    def getLaborDtlsByOperationSerialNum(self,serialNum):
        return store.fetchObjectsOfClass(LaborDtl,clauses="WHERE operationObjId=%s" % serialNum)

    def getLaborDtls(self):
        return store.fetchObjectsOfClass(LaborDtl)
   
    def getUnpostedLaborDtls(self, startStamp = None, endStamp = None):
        if not startStamp:
            startStamp = "1900-01-01 00:00:00"

        if not endStamp:
            endStamp = str(datetime.today())[:-7]

        return store.fetchObjectsOfClass(LaborDtl,"""WHERE posted=0 AND startStamp >= "%s" AND endStamp <= "%s" """ % ( startStamp, endStamp ) )
 
    def getLaborDtlsByDate(self):
        return store.fetchObjectsOfClass(LaborDtl,clauses="ORDER BY startStamp,operationObjId ASC")

    def getLaborDtlsByLaborHedSerialNum(self,serialNum):
        return store.fetchObjectsOfClass(LaborDtl,clauses="WHERE laborHedObjID='%s' ORDER BY startStamp ASC"% serialNum)

    def getLaborDtlHoursOnOperationBySerialNum(self,serialNum):
        timeWorked = 0
        empList = []
        for x in store.fetchObjectsOfClass(LaborDtl,clauses="WHERE operationObjId='%s' AND activeTrans = 0" % serialNum):

            time = ( x.endStamp() - x.startStamp() )
            if isinstance(timeWorked,int) and isinstance(x.startStamp(),datetime.datetime):
                timeWorked = datetime.timedelta()
            
            timeWorked += time
            empList += [ x.laborHed().employee().serialNum() ]
       
        if isinstance(timeWorked,datetime.timedelta):
            if hasattr(timeWorked,"days"):
                timeWorked = ( timeWorked.days * ( 24 * 3600 ) ) + timeWorked.seconds
            else:
                timeWorked = timeWorked.seconds
       
        return [ timeWorked, empList ]
