'''
LaborHed.py
'''

from Pyro.EventService.Clients import Publisher

from GeneratedPy.GenLaborHed import GenLaborHed
from sndcs_common.DateTimeMixin import DateTimeMixin
from sndcs_common.MathUtilMixin import MathUtilMixin
from sndcs.Config import config


class LaborHed(GenLaborHed, DateTimeMixin, MathUtilMixin, Publisher):

    def __init__(self):
        GenLaborHed.__init__(self)
        Publisher.__init__(self)
        self.eventservice._release()

        # Get the Pyro namespace from the config
        self.namespace = config.get("pyro", "namespace", "sndcs")






    ### Status Functions ###

    def getActiveLaborDetails(self):
        # This could maybe be optimized better
        return [dtl for dtl in self.laborDtls() if dtl.activeTrans() == 1] # This will return just active activities

    def getSuspendedLaborDetails(self):
        # This could maybe be optimized better
        return [dtl for dtl in self.laborDtls() if dtl.activeTrans() == 2] # This will return just suspended activities

    def getActiveAndSuspendedLaborDetails(self):
        # This could maybe be optimized better
        return [dtl for dtl in self.laborDtls() if dtl.activeTrans()] # This will return both active and suspended activities

    def getActiveProductionLaborDetails(self):
        return [dtl for dtl in self.laborDtls() if dtl.activeTrans() == 1 and dtl.jobType() == "Production"] # This will return just active activities

    def getActiveProductionAndSetupLaborDetails(self):
        return [dtl for dtl in self.laborDtls() if dtl.activeTrans() == 1 and dtl.jobType() in [ "Production", "Setup" ] ] # This will return just active activities
    
    def getNumberActiveProductionAndSetupLaborDetails(self):
        return len(self.getActiveProductionAndSetupLaborDetails())
    
    def getNumberActiveProductionLaborDetails(self):
        return len(self.getActiveProductionLaborDetails())

    def getNumberActiveLaborDtls(self):
        return len(self.getActiveLaborDetails())
    
    def isCurrentIndirectActivity(self, indirect):
        from Indirect import Indirect
        assert isinstance(indirect, Indirect)
        details = self.getActiveLaborDetails()
        for detail in details:
            if detail.indirect() is indirect:
                return True
        return False

    def isCurrentProductionActivity(self, operation, jobType="Production"):
        from Operation import Operation
        assert isinstance(operation, Operation)
        details = self.getActiveLaborDetails()
        for detail in details:
            if detail.operation() is operation and detail.jobType() == jobType:
                return True
        return False







    ### Ending activities ###

    def endAllLaborDtl(self, end_date_time = None, terminal_id = None, suspend = 0, scrap_codes = None):
        """End all active and/or suspended labordtl records.
        This method will auto allocate the activity percentage so it should only be called when user doesn't want to explicitly end records (i.e. clock out)"""
        labordtls = self.getActiveAndSuspendedLaborDetails()
        if not labordtls:
            return
        # Use the calculated percentage for every one except the last one
        # For the last one figure out what's left over
        #percentage = (math.floor(((100.0/float(len(labordtls)))*10)+0.5)) / 10
        #last_percentage = 100.0 -(percentage * (len(labordtls) - 1))
        ##activity_percentage = 100.0 / float(len(labordtls))
        percentage, last_percentage = self.calculateDefaultActivityPercentages(len(labordtls))
        i = 1
        for dtl in labordtls:
            if suspend:
                suspend_record = 1
            else:
                suspend_record = 0

            if i == len(labordtls):
                activity_percentage = last_percentage
            else:
                activity_percentage = percentage
            dtl.end(end_date_time = end_date_time, terminal_id = terminal_id, activity_percentage = activity_percentage, suspend = suspend_record, scrap_codes = scrap_codes)
            i += 1

        from Store import store # Couldn't put this up at the top because I was getting weird errors
        store.saveChanges()
    
    def endAllActiveLaborDtl(self,terminal_id = None): #NOTE: Only Indirect tasks should EVER get here
        labordtls = self.getActiveAndSuspendedLaborDetails()
        if not labordtls:
            return
        
        for dtl in labordtls:
            if dtl.activeTrans()==1:
                dtl.end(terminal_id = terminal_id)


    def end(self, end_date_time = None, terminal_id = None, scrap_codes = None):
        """Ends all labordtl and the laborhed"""
        #assert not self.employee() # A bit of a hack... an active labor hed wouldn't have the employee back reference filled in yet
        assert not self.endStamp()

        if not end_date_time:
            end_date_time = self.now()

        self.endAllLaborDtl(end_date_time = end_date_time, terminal_id = terminal_id, scrap_codes = scrap_codes)

        self.setEndStamp(end_date_time)
        if terminal_id:
            self.setEndTerminal(terminal_id)
        from Store import store
        store.saveChanges()
        





    ### Starting activities ###
    
    def startLaborDtl(self, operation = None, indirect = None, jobType = "Production", startStamp = None, startTerminal = None, notes = None):
        """
        Returns the newly created laborDtl for reference purposes.
        Just need to call store.saveChanges()
        """
        from LaborDtlFactory import LaborDtlFactory
        dtl = LaborDtlFactory().new(operation=operation, indirect=indirect, jobType=jobType, startStamp=startStamp, startTerminal=startTerminal, notes=notes)
        self.addToLaborDtls(dtl)
        # We'll let the calling function call store.saveChanges() because it might want to start several jobs
        # ... on second though... in order to send events the dtl record will need to have a serialNum so need to save now
        from Store import store
        store.saveChanges()
        employee = self.employee()
        if jobType == "Indirect":
            self.publish(self.namespace + "_indirect_start", {"serialNum":employee.serialNum(), "empId":employee.empId(), "properName":employee.properName(), "startStamp":startStamp, "startTerminal":startTerminal, "activity":{"serialNum":dtl.serialNum(), "indirect":indirect.code(), "description":indirect.formattedDescription()}})
            self.eventservice._release()
        elif jobType == "Production":
            self.publish(self.namespace + "_production_start", {"serialNum":employee.serialNum(), "empId":employee.empId(), "properName":employee.properName(), "startStamp":startStamp, "startTerminal":startTerminal, "activity":{"serialNum":dtl.serialNum(), "operation":operation.number(), "operation_id":operation.serialNum(), "description":operation.formattedDescription()}})
            self.eventservice._release()
        elif jobType == "Setup":
            self.publish(self.namespace + "_setup_start", {"serialNum":employee.serialNum(), "empId":employee.empId(), "properName":employee.properName(), "startStamp":startStamp, "startTerminal":startTerminal, "activity":{"serialNum":dtl.serialNum(), "operation":operation.number(), "operation_id":operation.serialNum(), "description":operation.formattedDescription()}})
            self.eventservice._release()
        return dtl







    ### Suspending and resuming activites ###

    def suspendAllLaborDtl(self):
        self.endAllLaborDtl(suspend = 1)

    def resumeAllLaborDtls(self, terminal_id = None):
        #TODO: We need to index the activeTrans field
        labordtls = self.getSuspendedLaborDetails()
        if not labordtls:
            return
        start_stamp = self.now()
        for dtl in labordtls:
            dtl.resume(terminal_id = terminal_id, start_stamp = start_stamp)

        from Store import store # Couldn't put this up at the top because I was getting weird errors
        store.saveChanges()



    ### start/stop/continue ###
    def startStopContinueLaborDtls(self, starts = None, stops = None, continues = None, terminal_id = None, jobType = "Production"):
        from Store import store # Couldn't put this up at the top because I was getting weird errors
        now = self.now()
        if stops:
            for x in stops:
                x["labordtl"].end(now, terminal_id, x["activity_percentage"], x["qty_complete"], x["qty_scrap"], notes = x["notes"], opr_complete = x["operation_complete"], completed_by = x["completed_by"],scrap_codes=x["scrap_codes"])
            store.saveChanges()
        if starts:
            from Operation import Operation
            from Indirect import Indirect
            for x in starts:
                #TODO: make sure this is a real job
                #TODO: make sure there are no suspended activities... need to close those first.
                #TODO: make sure employee is not already on this activity
                #assert isinstance(x["operation"], Operation) or isinstance(x["indirect"], Indirect)
                self.startLaborDtl(operation = x["operation"], indirect = x["indirect"], jobType = jobType, startStamp = now, startTerminal = terminal_id)
        if continues:
            for x in continues:
                new = x["labordtl"].clone()
                new.setPrevLaborDtl(None) # Clear it out in case it is set
                store.addObject(new)
                #new.setPrevLaborDtl(x["labordtl"])
                new.setStartTerminal(terminal_id)
                new.setStartStamp(now)
                store.saveChanges() # Unfortunately need to do this here so "new" gets a serialNum that we can send out with the sndcs_production_start event
                
                self._laborDtls = None # clear out cache... probably a better way to do this
                x["labordtl"].end(now, terminal_id, x["activity_percentage"])

                # Send out the start event
                employee = new.laborHed().employee()
                operation = new.operation()
                if new.jobType() == "Production": # Can't currently gang anything other than production so this check is probably irrelevant... but could change in the future
                    self.publish(self.namespace + "_production_start", {"serialNum":employee.serialNum(), "empId":employee.empId(), "properName":employee.properName(), "startStamp":now, "startTerminal":terminal_id, "activity":{"serialNum":new.serialNum(), "operation":operation.number(), "operation_id":operation.serialNum(), "description":operation.formattedDescription()}})
                    self.eventservice._release()
                elif new.jobType() == "Setup":
                    self.publish(self.namespace + "_setup_start", {"serialNum":employee.serialNum(), "empId":employee.empId(), "properName":employee.properName(), "startStamp":now, "startTerminal":terminal_id, "activity":{"serialNum":new.serialNum(), "operation":operation.number(), "operation_id":operation.serialNum(), "description":operation.formattedDescription()}})
                    self.eventservice._release()
        store.saveChanges()
