# vim: ts=4 et sts=4 sw=4 autoindent

import os, sys
from sndcs.Config import config
from sndcs_common.SndcsExceptions import *

class Store(object):

    def __init__(self):
        self.engine = None
        self.host = None
        self.user = None
        self.passwd = None
        self.store = None
        self.pinger = None

    def createStore(self):
        self.engine = config.get("database", "engine")
        self.host = config.get("database", "host")
        self.user = config.get("database", "user")
        self.passwd = config.get("database", "passwd")
        self.name = config.get("database", "name")
        if self.engine == "MySQL":
            from MiddleKit.Run.MySQLObjectStore import MySQLObjectStore
            store = MySQLObjectStore(host=self.host, user=self.user, passwd=self.passwd, db=self.name)
        elif self.engine == "PostgreSQL":
            from MiddleKit.Run.PostgreSQLObjectStore import PostgreSQLObjectStore
            store = PostgreSQLObjectStore(host=self.host, user=self.user, password=self.passwd, db=self.name)
        elif self.engine == "MSSQL":
            from MiddleKit.Run.MSSQLObjectStore import MSSQLObjectStore
            store=MSSQLObjectStore(dsn=self.host, user=self.user, password=self.passwd, clear_auto_commit=0)

        filename = os.path.join(sys.prefix, "share/sndcs/sndcs.mkmodel")
        assert os.path.exists(filename)
        try:
            store.readModelFileNamed(filename)
        except Exception, e: # May have to explicetly trap database errors (i.e. _mysql_exceptions.OperationalError)
            # Probably have improper database setup... or database is down
            raise SndcsStoreConnectionError("Could not connect to %s server at %s as user '%s'. Error: %s" % (self.engine, self.host, self.user, str(e)))

        store.sqlEscape = self.sqlEscape
        store.engine = self.engine

        self.store = store
                
        return self.store

    def sqlEscape(self, string):
        """ Use sqlEscape() to escape any string that will be used
        in a SQL statement (i.e. WHERE clauses). You must also be sure
        not to put quotes around the %s if you are using string formatting.
        i.e. clauses="WHERE description=%s" % (sql_escape(description)
        is the correct usage."""

        if self.engine == "MySQL":
            if string:
                import MySQLdb
                return MySQLdb.string_literal(string)
            else:
                return "NULL"
        else: # TODO: implement for other engines
            return string


store = Store().createStore()
