import Pyro.core
from sndcs.IndirectFactory import IndirectFactory
from sndcs_common.SndcsExceptions import *

from sndcs_common.Logger import logger
log = logger.getLogger("sndcsd")

class IndirectProxyFactory(Pyro.core.ObjBase):

    ### __init__ functions ###
    ### From IndirectFactory ###
    def getActiveIndirects(self):
        indirects = []
        ind = IndirectFactory().getActiveIndirects()
        for x in ind:
            indirects.append({"serialNum":x.serialNum(), "code":x.code(), "description":x.description()})
        return indirects


    def setIndirectByCode(self, code):
        indirect = IndirectFactory().getIndirectByCode(code)
        if not indirect:
            raise SndcsIndirectError("No indirect activity with code = '%s'" % (code))
        proxy = IndirectProxy()
        proxy.indirect = indirect
        self.getDaemon().connect(proxy)
        log.debug("Returning indirect proxy object %s.", proxy)
        return proxy.getProxy()

    def setIndirectBySerialNum(self, serialNum):
        indirect = IndirectFactory().getIndirectBySerialNum(serialNum)
        if not indirect:
            raise SndcsIndirectError("No indirect activity with serialNum = '%s'" % (serialNum))
        proxy = IndirectProxy()
        proxy.indirect = indirect
        self.getDaemon().connect(proxy)
        log.debug("Returning indirect proxy object %s.", proxy)
        return proxy.getProxy()

class IndirectProxy(Pyro.core.ObjBase):

    def getDataDict(self):
        """Utility to return indirect data in a dictionary"""
        return {"indirect_id":self.indirect_id(), "code":self.code(), "formattedDescription":self.formattedDescription(), "isActive":self.isActive()}



    ### Utilities ###
    ### From Indirect.py ###
    def formattedDescription(self):
        return self.indirect.formattedDescription()


    ### Getters and Setters ###
    def serialNum(self):
        return self.indirect.serialNum()
    def code(self):
        return self.indirect.code()
    def description(self):
        """ NOTE - should probably use formattedDescription() instead unless you really want this"""
        return self.indirect.description()
    def isActive(self):
        return self.indirect.isActive()

    def indirect_id(self):
        return self.indirect.serialNum()



    ### Cleanup ###
    def disconnect(self):
        daemon = self.getDaemon()
        log.debug("Disconnecting indirect proxy object %s.", self)
        daemon.disconnect(self)
