import Pyro.core
from sndcs.OperationFactory import OperationFactory
from sndcs_common.SndcsExceptions import *
from sndcs.Operation import formattedOperationDescription

from sndcs_common.Logger import logger
log = logger.getLogger("sndcsd")

class OperationProxyFactory(Pyro.core.ObjBase):

    def getAvailableActivitiesData(self):
        """
        Utility function to retrieve available activity data.
        Returns data structures representing available activities.
        """

        def _(data):
            if not data: return ""
            else: return data

        activities = []
        from sndcs.Store import store
        con, cur = store.executeSQL("SELECT opr.serialNum, opr.number, asm.number, job.number, job.itemNumber, job.itemDescription, asm.description, opr.description, job.priority, job.hidden, cust.customerId, cust.name, dept.code, tasks.code, tasks.description FROM Operation AS opr LEFT JOIN Assembly AS asm ON opr.assemblyObjId=asm.serialNum LEFT JOIN JobHed AS job ON asm.JobHedObjId=job.serialNum LEFT JOIN Customer AS cust ON job.customerObjId=cust.serialNum LEFT JOIN Department AS dept ON opr.departmentObjId=dept.serialNum LEFT JOIN Tasks AS tasks ON opr.tasksObjId=tasks.serialNum WHERE opr.complete=0 ORDER BY job.priority DESC")
        all = cur.fetchall()
        for x in all:
            if not x[9]:
                data = {"customer_id":_(x[10]), "customer_name":_(x[11]), "job_number":_(x[3]), "item_number":_(x[4]), "item_description":_(x[5]), "assembly_number":_(x[2]), "assembly_description":_(x[6]), "operation_number":_(x[1]), "operation_description":_(x[7]), "task_code":_(x[13]), "task_description":_(x[14])}
                activities.append({"job":x[3], "assembly":x[2], "operation":x[1], "operation_id":x[0], "description":formattedOperationDescription(data), "priority":x[8],"department":x[12]})
        return activities

    def setOperationBySerialNum(self, serialNum):
        operation = OperationFactory().getOperationBySerialNum(serialNum)
        if not operation:
            raise SndcsOperationError("No operation with serialNum = '%s'" % (serialNum))
        proxy = OperationProxy()
        proxy.operation = operation
        self.getDaemon().connect(proxy)
        log.debug("Returning operation proxy object %s.", proxy)
        return proxy.getProxy()

    def setOperationByJobAssemblyOperation(self, job_num, assembly_num = None, operation_num = None, separator = '-'):
        operation = OperationFactory().getOperationByJobAssemblyOperation(job_num, assembly_num, operation_num, separator)
        if not operation:
            raise SndcsOperationError()
        proxy = OperationProxy()
        proxy.operation = operation
        self.getDaemon().connect(proxy)
        log.debug("Returning operation proxy object %s.", proxy)
        return proxy.getProxy()


class OperationProxy(Pyro.core.ObjBase):

    def formattedDescription(self):
        return self.operation.formattedDescription()


    ### Getters and Setters
    def serialNum(self):
        return self.operation.serialNum()


    ### Cleanup ###
    def disconnect(self):
        daemon = self.getDaemon()
        log.debug("Disconnecting operation proxy object %s.", self)
        daemon.disconnect(self)
