# vim: ts=4 et sts=4 sw=4 autoindent

from sndcs.EmployeeFactory import EmployeeFactory
from sndcs.LaborDtlFactory import LaborDtlFactory
from sndcs.LaborHedFactory import LaborHedFactory
from sndcs.JobHedFactory import JobHedFactory
import datetime

class Reporting(object):
    
    def formattedReportString(self,*data):
        dataList = [str(x) for x in data ] 
        return "<tr width=100%%><td align=left><nobr>%s</nobr></td></tr>\n" % "</nobr></td><td align=left><nobr>".join( dataList ) #, len( dataList ) # Returns results and colspan

    def formatTime(self,data = None, seconds = 0):
        if not isinstance(data,int):
            seconds = data.seconds

            if isinstance(data,datetime.timedelta) and hasattr(data,"days"):
                seconds += data.days * (24 * 3600)
        else:
            seconds = data

        return int( round( seconds / 60 ) ) #Return minutes.
    
    def trueTime(self,data):
        if not data:
            return "N/A"
        elif data < 60:
            return "%s Minute%s" % ( data, self.ifTrueReturn(data == 1,"","s") )
        elif divmod(data,60)[1] == 0:
            result = data / 60  
            return "%s Hour%s" % ( result,self.ifTrueReturn(result == 1,"","s") )
        else:
            results = divmod(data,60)
            return "%s Hour%s, and %s Minute%s" % (results[0],self.ifTrueReturn(results[0] == 1,"","s"),results[1],self.ifTrueReturn(results[1] == 1,"","s") ) 

    def timeString(self,data):
        return self.trueTime( self.formatTime( data ) )

    def formatMoneyString(self,data):
        return "$%.2f" % data

    def assemblyString(self,data):
        colspan = 6
        return """<tr><td colspan=%s><hr width=100%%></td></tr>\n\t%s""" % (colspan,data)

    def getLoggedOnHoursForOperation(self,operation):
        output = LaborDtlFactory().getLaborDtlHoursOnOperationBySerialNum( operation.serialNum() )
        empList = self.parseList(output[1])
        money = ( self.formatTime( output[0] ) / 60.0 ) * operation.assembly().itemPrice()
        result = [ [ output[0], empList ], self.formattedReportString("Operation #%s: %s" % ( operation.number(),operation.description()), "Hours Worked: ", self.timeString( output[0] ), "Number Of Employees: %s" % len(empList), "Total: ", self.formatMoneyString(money) ), money]
        return result

    def getLoggedOnHoursForAssembly(self,assembly):
        timeWorked = 0
        empList = []
        reportList = []
        money = 0
        for x in assembly.operations():
            obj = self.getLoggedOnHoursForOperation( x )
            empList += obj[0][1]
            empList = self.parseList(empList)
            timeWorked += obj[0][0]
            reportList += obj[1]
            money += obj[2]

        output = [ [ timeWorked, empList ], [ self.assemblyString( self.formattedReportString( "Assembly #%s: %s" % (assembly.number(),assembly.description()), "Hours Worked: ", self.timeString(timeWorked), "Number Of Employees: %s" % len(empList),"Total: ", self.formatMoneyString(money) ) ), reportList ],money ]
        return output

    def getLoggedOnHoursForJobHed(self,jobHed):
        if isinstance(jobHed,int):
            jobHed = JobHedFactory().getJobHedBySerialNum( serialNum )

        timeWorked = 0
        empList = []
        reportList = []
        money = 0
        for x in jobHed.assemblies():
            obj = self.getLoggedOnHoursForAssembly( x )
            timeWorked += obj[0][0]
            empList += obj[0][1]
            empList = self.parseList(empList)
            reportList += obj[1]
            money += obj[2]
    
        return [ self.formattedReportString("Time Worked: ",self.trueTime( self.formatTime(timeWorked) ), "Number Of Employees: ", len(empList),"Total: ", self.formatMoneyString(money) ), reportList ]

    
    def getLoggedOnHoursForEmployee(self,serialNum,startStamp = None, endStamp = None):
        return LaborHedFactory().getLaborHedsHoursForEmployeeBySerialNum( serialNum, startStamp, endStamp )

    def parseList(self,data):
        output = []
        for x in data:
            if not output.count(x):
                output += [x]
        return output

    def getEmployeeReport(self,employee,startStamp = None, endStamp = None):
        if type(employee) == int: 
            employee = EmployeeFactory().getEmployeeBySerialNum(employee)

        report = self.getLoggedOnHoursForEmployee( employee.serialNum(), startStamp = startStamp, endStamp = endStamp )
        return ( """
            <tr><td class=body_desc><b>Employee Name:</b></td><td align=left>%s</td></tr>
            <tr><td class=body_desc><b>Hours Worked:</b></td><td align=left>%s</td></tr>
            <tr><td class=body_desc><b>Hours Worked In Last 7 Days:</b></td><td>%s</td></tr>
            <tr><td class=body_desc><b>LaborHeds:</b></td><td align=left> %s</td></tr>
            <tr><td class=body_desc><b>LaborDtls:</b></td><td align=left> %s</td></tr>
            <tr><td class=body_desc><b>Total Production Hours:</b></td><td align=left> %s</td></tr>
            <tr><td class=body_desc><b>Total Indirect Hours:</b></td><td align=left> %s</td></tr>
            <tr><td class=body_desc><b>Total Setup Hours:</b></td><td align=left> %s</td></tr>
        """ % (employee.properName(),self.timeString( report[0] ),self.timeString( report[3] ),report[1],report[2],self.timeString( report[4] ),self.timeString( report[5] ),self.timeString( report[6] ) ), report )

    def getGlobalEmployeeReport(self,startStamp = None, endStamp = None):
        reportList = []
        
        hoursTotal = 0
        productionHoursTotal = 0
        setupHoursTotal = 0
        indirectHoursTotal = 0
        hoursInLastWeek = 0
        laborHeds = 0
        laborDtls = 0

        for employee in EmployeeFactory().getEmployees():
            report = self.getEmployeeReport( employee, startStamp, endStamp )
            reportList += [ report[0] ]
            hoursTotal += self.formatTime(report[1][0]) * 60
            productionHoursTotal += self.formatTime(report[1][4]) * 60
            setupHoursTotal += self.formatTime(report[1][6]) * 60
            indirectHoursTotal += self.formatTime(report[1][5]) * 60
            hoursInLastWeek += self.formatTime( report[1][3] ) * 60
            laborHeds += report[1][1]
            laborDtls += report[1][2]

        return """
            <tr><td class=body_desc>Number Of Employees: </td><td>%s</td></tr>
            <tr><td class=body_desc>Total Hours: </td><td>%s</td></tr>
            <tr><td class=body_desc>Total Hours In Last 7 Days: </td><td>%s</td></tr>
            <tr><td class=body_desc>Total Production Hours: </td><td>%s</td>
            <tr><td class=body_desc>Total Indirect Hours: </td><td>%s</td>
            <tr><td class=body_desc>Total Setup Hours: </td><td>%s</td>
            <tr><td class=body_desc>LaborHeds:</td><td>%s</td></tr>
            <tr><td class=body_desc>LaborDtls:</td><td>%s</td></tr>
            <tr><td colspan=2 align=center><hr></td></tr>
        %s""" % (len(reportList),self.timeString(hoursTotal),self.timeString(hoursInLastWeek),self.timeString(productionHoursTotal),self.timeString(indirectHoursTotal),self.timeString(setupHoursTotal),laborHeds,laborDtls,"<tr><td align=center colspan=2><hr></td></tr>".join( reportList) )


    def generateList(self,data):
        dataString = ""
        if type(data) == list:
            for x in data:
                dataString += self.generateList( x )
        elif type(data) == tuple:
            dataString += data[0]
        else: dataString += str(data)

        return dataString

    def getJobHedReport(self,serialNum):
        jobHed = JobHedFactory().getJobHedBySerialNum(serialNum) 
        report = self.getLoggedOnHoursForJobHed( jobHed )
        reportList = self.generateList( report[1] )
        colspan = 6

        return """
            <table width=75%% align=center>
                <tr class=body_desc><td align=center colspan=%s> <b>JobHed #%s Report:</b> %s - %s</td></tr>
                 <tr>
                    <td colspan=%s>
                   <table align=center cellpadding=5 width=100%%>  
                   %s
                   %s
                   </table>
                    </td>
                </tr>
            </table>
        """ % ( colspan, jobHed.number(), jobHed.customer().name(),jobHed.itemDescription(), colspan, report[0], reportList )
