# vim: ts=4 et sts=4 sw=4 autoindent

from WebKit.Page import Page
from sndcs_common import VERSION
from os.path import exists as _fileExists
import datetime
from sndcs.Config import config
from ConfigParser import NoOptionError
import xml.sax.saxutils

class SitePage(Page):
        
    """Scripts that the Page needs"""
    mainScripts = [ "../js/sndcs2_main.js" ]
    
    def __init__(self,currentTab,subtab=0,meta_refresh=False,scripts=[ "../js/sndcs2_main.js" ],clearSearchResults=True):
        Page.__init__(self)
        
        """Very important for javascript scripts"""
        self.currentTab = currentTab
        self.subtab = subtab
        
        self.meta_refresh = meta_refresh
        self.clearSearchResults = clearSearchResults
        
        """Tab Width is 92px"""
        self.tab_width = 92

        self.scripts=scripts
        self.newObject= [ "","" ]

        self.terminalName = config.get("web","terminal_id","sndcs_web")

        self.namespace = config.get("pyro","namespace","sndcs")
        
        self.message = ""

    def siteRoot(self):
        return self.request().siteRoot()

    def getBackPage(self):
        if not self.backPage():
            return self.session().value("backPage")
        else: return self.backPage()

    def goToPage(self, page=False, absolute=False):
        """ Uses sendRedirectAndEnd so it will end the response """
        if not page: page=self.backPage()
        if absolute is False:
            self.sendRedirectAndEnd(self.siteRoot() + page)
        else:
            self.sendRedirectAndEnd(page)

    def createImageTag(self, image_name, extraParams=""):
        return """<img src="%simages/%s" border=0 %s>""" % (self.siteRoot(), image_name, extraParams)

    def currentdate(self):
        return datetime.datetime.today().strftime("%m/%d/%Y")

    def parseDate(self,date,default = "N/A"):
        date = str(date)[:10].split("-")
        if not len(date) == 3:
            return default
        
        date = "%s/%s/%s" % (date[1],date[2],date[0])
        return date

    def escape(self, string, default = ""):
        """ Use escape() to escape any string that will be used in
        HTML that is *NOT* a form field"""

        if string:
            if isinstance(string, str):
                return xml.sax.saxutils.escape(string)
            else:
                return string
        else:
            return default

    def clear(self,object):
        for x in object.allAttrs():
            setattr(object,x,None)

    def isType(self,object,typeList ):
        if not isinstance(typeList,list):
            typeList = [ typeList ]

        for x in typeList:
            try:
                return x(object)
            except (TypeError,ValueError): continue

        return False

    def returnType(self,value,typeOf,default = ""):
        value = self.isType(value,typeOf)
        if value:
            return value
        else:
            return default

    def checkBoolean(self,string):
        if string in [ False, "False" ]:
            return False
        elif string in [ True, "True" ]:
            return True
        else:
            return string

    def ifTrueReturn(self,value,yes,no = ""):
        if value:
            return yes
        else:
            return no

    def returnTime(self,time): # Hack around MX.DateTime and datetime clash...
        if isinstance(time,str):
            time = time.split(":")
            if len(time[0]) < 2:
                time[0] = "0" + str(time[0])
            return ":".join(time)
        elif time:
            return time()

    def checkLength(self,value,size,default = None):
        if len(value) == size:
            return value
        else:
            return default
    
    def middleKitObjectEval(self,object,key,value,type = None):
        try:
            if not type:
                eval("""object.set%s(\"\"\"%s\"\"\")""" % (key,self.escape(value) ) )
            elif type == "Boolean":
                eval("""object.set%s(%s)""" % (key,self.escape(value) ) )
        except TypeError:
            eval("""object.set%s(None)""" % key )

    def quoted_escape(self, string, default = ""):
        """ Use quoted_escape() to escape any string that will be used in
        HTML that *IS* a form field (except for textareas, use escape instead). You must also
        be sure not to put quotes around the %s if you are using string formatting.  i.e.
        '<input type="text" name="description" value=%s>' % (quoted_escape(type.description()))
        is the correct usage."""

        if string:
            if isinstance(string, str):
                return xml.sax.saxutils.quoteattr(string)
            else:
                return string
        else:
            return default

    def convertToDateTime(self, timestamp = False, date=""):
            if timestamp == "N/A" or date == "N/A":
                return False #Incomplete LaborDtls pass this.
        
            if timestamp:
                if len(timestamp.split()) == 2:
                    [date,time] = timestamp.split()
                    date = date.split("/")
                    time = time.split(":")
                else:
                    time = timestamp.split(":")
                    date = ""
            else:
                date = date.replace("/","-")
                date = date.split("-")
                time = ""

            if len(date) == 3 and len(time) == 3:
                try:
                    return datetime.datetime(int(date[2]),int(date[0]),int(date[1]),int(time[0]),int(time[1]),int(time[2])) # Year, Month, Day, Hour, Minute, Second
                except ValueError: pass
                
            if len(date) == 3 and not len(time):
                try:
                    return datetime.date(int(date[2]),int(date[0]),int(date[1]))
                except ValueError: pass
                
            if not len(date) and len(time) == 3:
                try:
                    return datetime.time(int(time[0]), int(time[1]), int(time[2]))
                except ValueError: pass
            return False #NOTE: Invalid datetime input

    def returnTrueOrDefault(self,value,default):
        if value:
            return value
        else:
            return default

    def redirectIfTrue(self,value):
        if value:
            self.goToPage(value)
        else:
            return False

    def isAdministrator(self):
        if self.session().value("userLoggedOn",0) == 1:
            return True
        else:
            return False

    def htBodyArgs(self):
        if not self.currentTab==0:
            return """onLoad='setState(%d,%d);'""" % (self.currentTab,self.subtab)
    
    def write_option(self,value,caption,selected=False):
        selected = self.ifTrueReturn(selected,"SELECTED","")
        return ("\t<option value='%s' %s>%s</option>" % (value,selected,caption) )
    
    def state_option(self,selected=''):
        states = {
        "AL" : "Alabama",
        "AK" : "Alaska",
        "AZ" : "Arizona",
        "AR" : "Arkansas",
        "CA" : "California",
        "CO" : "Colorado",
        "CT" : "Connecticut",
        "DE" : "Delaware",
        "FL" : "Florida",
        "GA" : "Georgia",
        "HI" : "Hawaii",
        "ID" : "Idaho",
        "IL" : "Illinois",
        "IN" : "Indiana",
        "IA" : "Iowa",
        "KS" : "Kansas",
        "KY" : "Kentucky",
        "LA" : "Louisiana",
        "ME" : "Maine",
        "MD" : "Maryland",
        "MA" : "Massachusetts",
        "MI" : "Michigan",
        "MN" : "Minnesota",
        "MS" : "Mississippi",
        "MO" : "Missouri",
        "MT" : "Montana",
        "NE" : "Nebraska",
        "NV" : "Nevada",
        "NH" : "New Hampshire",
        "NJ" : "New Jersey",
        "NM" : "New Mexico",
        "NY" : "New York",
        "NC" : "North Carolina",
        "ND" : "North Dakota",
        "OH" : "Ohio",
        "OK" : "Oklahoma",
        "OR" : "Oregon",
        "PA" : "Pennsylvania",
        "RI" : "Rhode Island",
        "SC" : "South Carolina",
        "SD" : "South Dakota",
        "TN" : "Tennessee",
        "TX" : "Texas",
        "UT" : "Utah",
        "VT" : "Vermont",
        "VA" : "Virginia",
        "WA" : "Washington",
        "WV" : "West Virginia",
        "WI" : "Wisconsin",
        "WY" : "Wyoming"
        }

        states=states.items()
        states.sort()
        
        output=""

        for abbev,state in states:
            if selected==abbev:
                output += ( self.write_option(abbev,state,True) )
            else:
                output += ( self.write_option(abbev,state,False) )

        return output
        
    def start_color(self):
        self.number = 0

    def next_color(self):
        self.number+=1
        if (self.number % 2):
            bgcolor="#ffffff"
        else:
            bgcolor="#c0c0c0"
        return bgcolor
    
    def write_div(self,id,classname,text,construct):
        subtab,tab,url = construct
        return ("""<div id="%s" class="%s" style="width: %dpx; z-index: 0;" onClick="redirectUrl('%s');setState(%d,%d);" onMouseover="hover(this)" onMouseout="setState(%d,%d)">%s</div>
        """ % (id,classname,self.tab_width,url,tab,subtab,tab,subtab,text))# Return <DIV>

    def localTitle(self):
        """ Subclasses should override this with an appropriate title for the page. """
        return "SaberNet DCS"

    def title(self):
        return "SaberNet DCS " + VERSION + ": "+ self.localTitle()
        
    def writeStyleSheet(self,linkStyle=False):
        self.writeln("""<link rel="stylesheet" href="%scss/sndcs2.css" type="text/css"/>""" % (self.siteRoot()))
        if linkStyle:
             self.writeln("""
             <style>
                    .table_header a {
                      color:#ffffff;
                      border-width: 0px;
                      text-decoration:none;
                    }
                    .table_header a:link {
                      color:#ffffff;
                      text-decoration:none;
                    }
                    .table_header a:hover {
                      color:#9999CC;
                      text-decoration:none;
                    }
                    .table_header a:active {
                      color:#ffffff;
                      text-decoration:none;
                    }
            }
            </style>""")




    def writeHead(self):
        self.writeln("<head>")
        self.writeln("\t<title>%s</title>" % self.title())
        self.writeStyleSheet()
        if (self.meta_refresh):
            self.writeln("\t<meta http-equiv='REFRESH' content='30'/>")
        
        for scriptName in self.scripts:
            self.writeln("<script src='%s'></script>" % scriptName)
        
        if self.clearSearchResults:
            self.session().setValue("searchResults",False)   
        elif self.session().value("searchResults",False):
            self.subtab=2

        if self.subtab:
            subTabString="\t\n\tvar currentSubTab=%d;" % self.subtab
        else:
            subTabString=""
        
        self.writeln("""<script language="javascript">\n\tvar currentTab=%d;%s\n</script>\n</head>""" % (self.currentTab,subTabString))

    def convertToHexString(self,string):
        result = ""
        for eachByte in string:
            hexValue = str( hex( ord( eachByte ) ) ).replace("0x","%")
            result += hexValue
        return result
        
    def writeLocalContent(self):
        raise NotImplementedError, self.__class__

    def newTab(self):
        if self.session().value("searchResults",False) and not self.clearSearchResults:
            return [ "Back To Search Results", self.session().value("searchResults") ]
        else:
            return [ "", "" ]

    def backPage(self):
        """
        Page must override this and return the full path to the back page if they wish to provide this functionality.
        For example: return self.siteRoot() + "dcs_login/login.py"
        """
        return None

    def setBackPage(self):
        if self.backPage():
            self.session().setValue("backPage", self.backPage())
    
    def getSearchResultsPage(self):
        return self.session().value("searchResults",False)

    def setSearchResultsPage(self):
        self.session().setValue("searchResults",self.session().value("backPage") )
    
    def writeContent(self, message="", newObject= [ "", ""] ):
        """ message is so actions can call writeContent and pass a message that writeLocalContent can do whatever with"""
        
        if self.message == "":
            self.message = message

        if self.newObject == "":
            self.newObject = newObject

        if self.getSearchResultsPage():
            self.newObject= [ "Back To Search Results", "../%s" % self.getSearchResultsPage() ]

        self.setBackPage()

        # Display <noscript> if javascript isn't enabled
        self.writeln("""
        <noscript>
        <meta http-equiv='refresh' content='0; URL=../dcs_login/no_js.py'>
        </noscript>
        """)

        self.writeln("""<table CLASS="main_table" cellpadding=0 cellspacing=0>""") # Main layout table
        self.writeln("""<tr><td align="left" width="31px">%s</td><td CLASS="top_edge">&nbsp;</td><td align="right" width="29px">%s</td></tr>""" % (self.createImageTag("tl_corner.jpg"), self.createImageTag("tr_corner.jpg")))
        
        # Would do a foreach loop here but wouldn't be any faster or more readable
        # Employee_list tab
        tab1=self.write_div(id="tab1",classname="tab",text="Main", construct= [ 0, 1, "../dcs_employee_list/index.py" ]) 
        # Time tab
        tab2 = self.write_div(id="tab2",classname="tab",text="Time", construct = [ 0, 2, "../dcs_edit_time/index.py" ])
        # Employee_List tab
        tab3 = self.write_div(id="tab3",classname="tab",text="Employees", construct = [ 0, 3, "../dcs_edit_employees/index.py" ])
        # Shifts tab
        tab4 = self.write_div(id="tab4",classname="tab",text="Shifts", construct = [ 0, 4, "../dcs_edit_shifts/index.py" ])
        # Indirects tab     
        tab5 = self.write_div(id="tab5",classname="tab",text="Indirects", construct = [ 0, 5, "../dcs_edit_indirect/index.py" ])
        # Departments tab     
        tab6 = self.write_div(id="tab6",classname="tab",text="Departments", construct = [ 0, 6, "../dcs_edit_departments/index.py" ])
        # Jobs tab
        tab7 = self.write_div(id="tab7",classname="tab",text="Jobs", construct = [ 0, 7, "../dcs_edit_jobs/search.py" ])
        # Items tab
        tab8 = self.write_div(id="tab8",classname="tab",text="Items", construct = [ 0, 8, "../dcs_edit_items/index.py" ])
        # Customers tab
        tab9 = self.write_div(id="tab9",classname="tab",text="Customers", construct = [ 0, 9, "../dcs_edit_customers/index.py" ])
        # Admin tab
        tab10=self.write_div(id="tab10",classname="tab",text="Admin", construct = [ 0, 10, "../dcs_admin/index.py" ])

        self.writeln("""<tr><td CLASS="left_edge">&nbsp;</td><td align="center"><table width="100%">""")
        self.writeln("""<tr><td><table width="100%%"><tr><td align="left" width="346px">%s</td><td CLASS="header_text" align="center">%s</td></tr></table></td></tr>""" % (self.createImageTag("dcs_logo.jpg"), self.localTitle()))
        
        divisor = self.ifTrueReturn( self.newObject[0],"|","" )

        if self.currentTab:
            self.writeln("""<tr><td align='right' CLASS='refresh'><a href='%s'>%s</a> %s <a href='../dcs_login/login.py?_action_=Logout'>Logout</a> | <a href='./index.py'>Refresh</a></td> </tr>""" % (self.newObject[1],self.newObject[0],divisor ) )
            self.writeln("""<tr><td><table><tr><td>%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td></tr></table></td></tr>""" % (tab1,tab2,tab3,tab4,tab5,tab6,tab7,tab8,tab9,tab10))
        self.writeln("""<tr><td align="center">""")
        self.writeLocalContent()
        self.writeln("""</td></tr>""")
        self.writeln("""</table></td><td CLASS="right_edge">&nbsp;</td></tr>""")
        self.writeln("""<tr><td align="left">%s</td><td CLASS="bottom_edge">&nbsp;</td><td align="right">%s</td></tr>""" % (self.createImageTag("bl_corner.jpg"), self.createImageTag("br_corner.jpg")))
        self.writeln("""</table>""") # End main table

    def writeBody(self):
        self.newObject = self.newTab()
        Page.writeBody(self)
        self.message=""
        self.newObject=[ "", "" ]
