# vim: ts=4 et sts=4 sw=4 autoindent

from WebKit.Page import Page
from sndcs_common import VERSION
from os.path import exists as _fileExists
import datetime
from sndcs.Config import config
from ConfigParser import NoOptionError

class SitePage(Page):
        
    
    def __init__(self,meta_refresh=False):
        Page.__init__(self)
        
        self.meta_refresh=meta_refresh
        
        self.newObject= [ "","" ]

        #Get the terminal name
        self.terminal_name =  config.get("web","terminal_id","wap")

        #Get the namespace name
        self.namespace = config.get("pyro","namespace","sndcs")

        self.message=""

    def siteRoot(self):
        return self.request().siteRoot()

    def getBackPage(self):
        if not self.backPage():
            return self.session().value("backPage")
        else: return self.backPage()

    def goToPage(self, page=False, absolute=False):
        """ Uses sendRedirectAndEnd so it will end the response """
        if not page: page=self.backPage()
        if absolute is False:
            self.sendRedirectAndEnd(self.siteRoot() + page)
        else:
            self.sendRedirectAndEnd(page)

    def createImageTag(self, image_name, extraParams=""):
        return """<img src="%simages/%s" border=0 %s>""" % (self.siteRoot(), image_name, extraParams)

    def currentdate(self):
        return datetime.datetime.today().strftime("%m/%d/%Y")

    def start_color(self):
        self.number = 0

    def next_color(self):
        self.number+=1
        if (self.number % 2):
            bgcolor="#c0c0c0"
        else:
            bgcolor="#ffffff"
        return bgcolor
    
    def localTitle(self):
        """ Subclasses should override this with an appropriate title for the page. """
        return "SaberNet DCS"

    def title(self):
        return "SaberNet DCS " + VERSION + " WAP: "+ self.localTitle()
        
    def writeStyleSheet(self,linkStyle=False):
        self.writeln("""<link rel="stylesheet" href="%scss/sndcs2.css" type="text/css"/>""" % (self.siteRoot()))
        if linkStyle:
             self.writeln("""
             <style>
                    .table_header a {
                      color:#ffffff;
                      border-width: 0px;
                      text-decoration:none;
                    }
                    .table_header a:link {
                      color:#ffffff;
                      text-decoration:none;
                    }
                    .table_header a:hover {
                      color:#9999CC;
                      text-decoration:none;
                    }
                    .table_header a:active {
                      color:#ffffff;
                      text-decoration:none;
                    }
            }
            </style>""")

    def writeHead(self):
        self.writeln("<head>")
        self.writeln("\t<title>%s</title>" % self.title())
        self.writeStyleSheet()
        if (self.meta_refresh):
            self.writeln("\t<meta http-equiv='REFRESH' content='30'/>")
        self.writeln("\t<meta http-equiv='Pragma' content='no-cache'/>")
        self.writeln("\t<meta http-equiv='Expires' content='-1'/>")
        

    def writeLocalContent(self):
        raise NotImplementedError, self.__class__

    def backPage(self):
        """
        Page must override this and return the full path to the back page if they wish to provide this functionality.
        For example: return self.siteRoot() + "dcs_login/login.py"
        """
        return None

    def setBackPage(self):
        if self.backPage():
            self.session().setValue("backPage", self.backPage())
    
    def getSearchResultsPage(self):
        return self.session().value("searchResults",False)

    def setSearchResultsPage(self):
        self.session().setValue("searchResults",self.session().value("backPage") )
    
    def writeContent(self, message="", newObject= [ "", ""] ):
        """ message is so actions can call writeContent and pass a message that writeLocalContent can do whatever with"""
        
        if self.message=="":
            self.message = message

        if self.newObject=="":
            self.newObject = newObject

        if self.getSearchResultsPage():
            self.newObject= [ "Back To Search Results",self.getSearchResultsPage() ]

        self.setBackPage()

        self.writeln("""<table class="main_table" cellpadding=0 cellspacing=0>""") # Main layout
        self.writeln("""<tr><td align=center class="table_header">%s</font></td></tr>""" % self.localTitle())

        self.writeln("""<tr><td align="center">""")

        self.writeLocalContent()

        self.writeln("""</td></tr>""")

    def writeBody(self):
        Page.writeBody(self)
        self.message=""
        self.newObject=[ "", "" ]
