# vim: ts=4 et sts=4 sw=4 autoindent

from lib.SecureSitePage import SecureSitePage
from index import index

from etc import siteindex
from etc import salt

from sndcs.UserFactory import UserFactory
from sndcs.UserParameterFactory import UserParameterFactory
from sndcs.Store import store
import md5

class edit_user(index,SecureSitePage):

    def __init__(self):
        SecureSitePage.__init__(self,siteindex["dcs_admin"],5)
        self.value = 0
        self.username = ""

    def actions(self):
        return SecureSitePage.actions(self) + ["edit"]

    def writeJavascript(self):
        return """
        <script language="javascript">
        function selectAll(myCheckBox,myForm) {
        var myArray = new Array();

        myArray[0]="create_employee";
        myArray[1]="edit_employee";
        myArray[2]="delete_employee";
        
        myArray[3]="create_time";
        myArray[4]="edit_time";
        myArray[5]="delete_time";

        myArray[6]="create_indirect";
        myArray[7]="edit_indirect";
        myArray[8]="delete_indirect";

        myArray[9]="create_jobs";
        myArray[10]="edit_jobs";
        myArray[11]="delete_jobs";

        myArray[12]="create_items";
        myArray[13]="edit_items";
        myArray[14]="delete_items";

        myArray[15]="create_shifts";
        myArray[16]="edit_shifts";
        myArray[17]="delete_shifts";

        myArray[18]="create_departments";
        myArray[19]="edit_departments";
        myArray[20]="delete_departments";

        myArray[21]="create_customers";
        myArray[22]="edit_customers";
        myArray[23]="delete_customers";
       
        myArray[24]="admin_globals";

        for (x in myArray) {
            if (myCheckBox.checked == true) {
                (eval("myForm." + myArray[x])).checked = true;
            }
            else {
                (eval("myForm." + myArray[x])).checked = false;
            }
        }

        }
        </script>"""



    def edit(self):
        req = self.request()
        username = str(req.field("EditUserName",False))
        password = str(req.field("beginpassword","keepOld"))
        retypepassword = str(req.field("retypepassword","keepOld"))
        
        deleteValues = [ "x","y","beginpassword","retypepassword","EditUserName","_action_","serialNum" ]
        serialNum = self.returnType( req.field("serialNum",""), int, None)

        myvalue = 0
        for myfield in dict(req.fields()).items():
            if not myfield[0] in deleteValues:
                myvalue += int(UserFactory.priv_table[myfield[0]])

        if not password == retypepassword and not password == "keepOld" and not retypepassword == "keepOld": 
            self.message = "<b>Error:</b> Passwords do not match"
            self.writeBody()
            return
        
        elif serialNum:
            tmpUser = UserFactory().getUserBySerialNum(int(serialNum) )
            tmpParameter = UserParameterFactory().getUserParameterByName(tmpUser.username() )[0]
            tmpParameter.setValue(str(myvalue))
        else:
            if not UserFactory().getUserByName(self.escape( str(username) ) ):
                tmpUser = UserFactory().new()
                tmpParameter = UserParameterFactory().new()
                tmpUser.setCreateDate(self.convertToDateTime(date = self.currentdate()))

                tmpParameter.setAccount(tmpUser)
            else:
                self.message = "<b>Error:</b> User already exists"
                self.value = myvalue
                self.username=username
                self.writeBody()
                return
            
        tmpUser.setUsername( str(username) )
        if retypepassword == "keepOld": password="keepOld"
        if not password == "keepOld":
            tmpUser.setPassword("%s%s" % ( salt,str(md5.new(password).hexdigest() ) ) )     
        
        tmpParameter.setName(str(username) )
        tmpParameter.setValue(str(myvalue) )

        store.addObject(tmpParameter)
        store.addObject(tmpUser)
        store.saveChanges()
                                
        self.goToPage("dcs_admin/manage_user")

    def localTitle(self):
        req=self.request()
        if req.hasField("serialNum"):
            return "Edit User"
        else:
            return "Create New User"

    def newTab(self):
        return [ "Create New User", "edit_user" ]

    def writeLocalContent(self):
        req = self.request()
        serialNum = self.returnType(req.field("serialNum",""), int, None)

        priv_dictionary = dict(UserFactory.priv_table)
        for k,v in priv_dictionary.items():
            priv_dictionary[k]=""

        if serialNum:
            passwordTest = "keepOld"
            username = UserFactory().getUserBySerialNum(serialNum).username()
            value = int( UserParameterFactory().getUserParameterByName(username)[0].value() )
            usermode="READONLY"
        else:
            passwordTest=""
            username=self.username
            usermode=""
            value = self.value
                
            self.username =""
            self.value = 0

        if value:
            for k,v in UserFactory.priv_table.items(): 
                if (value & v) > 0: priv_dictionary[k]="CHECKED"

        self.writeln("""<table width="100%">""") # Password fields have the value of "keepOld" to keep the password the same.
        
        self.writeln( self.subTabs() )

        self.writeln("""<tr align='center'><td colspan=4><font color='#ff0000'>%s</font></td></tr>""" % self.message) 

        self.writeln("""
            <form action="./edit_user?_action_=edit" method="post">
            <tr class=table_header><th>User Parameters</th><th colspan=3></th></tr>    
            <tr><td colspan=4 align=center class=body_desc align=left>Username: <input type=text name=EditUserName value="%s" %s></td></tr>
        """ % (username,usermode) )
        
        if serialNum:
                self.writeln("""<input type="hidden" name="serialNum" value="%s">""" % serialNum)
        
        self.writeln("""<tr><td colspan=4 align=center><hr></td></tr>
        <tr><td class=body_desc align=left>Password:&nbsp;&nbsp;<input type=password name=beginpassword value='%s'>&nbsp;&nbsp;&nbsp;Retype-Password:&nbsp;&nbsp;<input type=password name=retypepassword value='%s'></td><td class=body_desc colspan=4 align=right style='text-align: center;'>Select All <input type='checkbox' onChange='selectAll(this,this.form);'></td></tr>""" % (passwordTest,passwordTest) )
        
        self.writeln("""<tr class=table_header><th width=80%%>Operation Type</th><th>Create</th><th>Edit</th><th>Delete</th></tr>
        <tr><td class=body_desc align=left>Employee Operations</td><td align=center><input type=checkbox name=create_employee value=1 %s></td><td align=center><input type=checkbox name=edit_employee value=1 %s></td><td align=center><input type=checkbox name=delete_employee value=1 %s></td></tr>""" % (priv_dictionary["create_employee"],priv_dictionary["edit_employee"],priv_dictionary["delete_employee"]) )
                
        self.writeln("""<tr><td class=body_desc align=left>Time Operations</td><td align=center><input type=checkbox name=create_time value=1 %s></td><td align=center><input type=checkbox name=edit_time value=1 %s></td><td align=center><input type=checkbox name=delete_time value=1 %s></td></tr>""" % (priv_dictionary["create_time"],priv_dictionary["edit_time"],priv_dictionary["delete_time"]) )
                
        
        self.writeln("""<tr><td class=body_desc align=left>Indirect Operations</td><td align=center><input type=checkbox name=create_indirect value=1 %s></td><td align=center><input type=checkbox name=edit_indirect value=1 %s></td><td align=center><input type=checkbox name=delete_indirect value=1 %s></td></tr>""" % (priv_dictionary["create_indirect"],priv_dictionary["edit_indirect"],priv_dictionary["delete_indirect"]) )

        self.writeln("""<tr><td class=body_desc align=left>Jobs Operations</td><td align=center><input type=checkbox name=create_jobs value=1 %s></td><td align=center><input type=checkbox name=edit_jobs value=1 %s></td><td align=center><input type=checkbox name=delete_jobs value=1 %s></td></tr>""" % (priv_dictionary["create_jobs"],priv_dictionary["edit_jobs"],priv_dictionary["delete_jobs"]) )
        
        self.writeln("""<tr><td class=body_desc align=left>Items Operations</td><td align=center><input type=checkbox name=create_items value=1 %s></td><td align=center><input type=checkbox name=edit_items value=1 %s></td><td align=center><input type=checkbox name=delete_items value=1 %s></td></tr>""" % (priv_dictionary["create_items"],priv_dictionary["edit_items"],priv_dictionary["delete_items"]) )

        
        self.writeln("""<tr><td class=body_desc align=left>Shift Operations</td><td align=center><input type=checkbox name=create_shifts value=1 %s></td><td align=center><input type=checkbox name=edit_shifts value=1 %s></td><td align=center><input type=checkbox name=delete_shifts value=1 %s></td></tr>""" % (priv_dictionary["create_shifts"],priv_dictionary["edit_shifts"],priv_dictionary["delete_shifts"]) )

        
        self.writeln("""<tr><td class=body_desc align=left>Departments Operations</td><td align=center><input type=checkbox name=create_departments value=1 %s></td><td align=center><input type=checkbox name=edit_departments value=1 %s></td><td align=center><input type=checkbox name=delete_departments value=1 %s></td></tr>""" % (priv_dictionary["create_departments"],priv_dictionary["edit_departments"],priv_dictionary["delete_departments"]) )

        self.writeln("""<tr><td class=body_desc align=left>Customers Operations</td><td align=center><input type=checkbox name=create_customers value=1 %s></td><td align=center><input type=checkbox name=edit_customers value=1 %s></td><td align=center><input type=checkbox name=delete_customers value=1 %s></td></tr>""" % (priv_dictionary["create_customers"],priv_dictionary["edit_customers"],priv_dictionary["delete_customers"]) )
        
        self.writeln("""<tr class=table_header><th>Global Operations</th><th colspan=2></th><th align=center>Allow&nbsp;</th></tr>
                <tr><td class=body_desc colspan=3 align=left>Admin Global Operations</td><td align=center><input type=checkbox name=admin_globals value=1 %s></td></tr>""" % priv_dictionary["admin_globals"])

        
        self.writeln("""
        <tr height='40px'><td colspan=4 align=center>
        <INPUT TYPE=image SRC="../images/submit1.gif" onMouseDown="this.src='../images/submit1down.gif';" onMouseOut="this.src='../images/submit1.gif';">
        &nbsp;<A HREF="./manage_user.py"><IMAGE SRC="../images/back1.gif" onMouseDown="this.src='../images/back1down.gif';" onMouseOut="this.src='../images/back1.gif';" BORDER=0></A></td></tr>
            </form>
        </table>""")
        
        self.writeln( self.writeJavascript() )


