# vim: ts=4 et sts=4 sw=4 autoindent

from lib.SecureSitePage import SecureSitePage
from etc import siteindex

from sndcs.LastUsedFactory import LastUsedFactory
from sndcs.AdminGlobalFactory import AdminGlobalFactory
from sndcs.UserFactory import UserFactory
from sndcs.Store import store

class index(SecureSitePage):
    
    def __init__(self,subTab=1):
        SecureSitePage.__init__(self,siteindex["dcs_admin"],subTab,False,( SecureSitePage.mainScripts + [ "../js/sndcs2_admin.js" ] ) )
        self.admin = AdminGlobalFactory().adminGlobal()

    def subTabs(self,colspan=1):
        subtab1 = self.ifTrueReturn( UserFactory().checkPriv( self.session().value("userLoggedOn"), "admin_globals" ), self.write_div(id="subtab1",classname="subtab",text="Main", construct = [ 1, siteindex["dcs_admin"], "index" ]), "" )
        
        subtab2 = self.write_div(id="subtab2",classname="subtab",text="User Preferences", construct = [ 2, siteindex["dcs_admin"], "user_preferences" ])
        subtab3 = self.write_div(id="subtab3",classname="subtab",text="Change Password", construct = [ 3, siteindex["dcs_admin"], "change_pass" ])
        subtab4 = self.write_div(id="subtab4",classname="subtab",text="Terminals", construct = [ 4, siteindex["dcs_admin"], "terminals" ])
        subtab5 = self.ifTrueReturn( self.isAdministrator(), self.write_div(id="subtab5",classname="subtab",text="Manage Users", construct = [ 5, siteindex["dcs_admin"], "manage_user" ]), "" )
        #subtab6 = self.ifTrueReturn( UserFactory().checkPriv( self.session().value("userLoggedOn"), "admin_globals" ), self.write_div(id="subtab6",classname="subtab",text="Reporting", construct = [ 6, siteindex["dcs_admin"], "reports" ] ), "" )
        subtab6 = ""

        return "<tr><td colspan='%s'><table align='left'><tr><td>%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td></tr></table></td></tr>" % (colspan,subtab1,subtab2,subtab3,subtab4,subtab5,subtab6)

    def actions(self):
        return SecureSitePage.actions(self) + [ "change" ]

    def change(self):
        req = self.request()
        self.redirectIfTrue( UserFactory().redirectIfAccessDenied( self.session().value("userLoggedOn"), "admin_globals" ) ) 
        deleteValues = [ "x","y","_action_","nextfield","last_used" ]
        self.clear(self.admin)

        last_used = self.returnType(req.field("last_used",""),int,None)
        if last_used:
            LastUsedFactory().setNextJobNumber( last_used - 1 )

        if req.field("JobNumbering",""):
                self.admin.setJobNumbering( self.ifTrueReturn("user" == req.field("JobNumbering"), "user", "auto" ) )
                deleteValues.append("JobNumbering")
        else:
                self.admin.setJobNumbering("user")
        
        if req.field("Prefix",""):
            self.admin.setPrefix( req.field("Prefix") )
            deleteValues.append("Prefix")
        else:
            self.admin.setPrefix("")

        if req.field("Suffix",""):
            self.admin.setSuffix( req.field("Suffix") )
            deleteValues.append("Suffix")
        else:
            self.admin.setSuffix("")
        
        for key,value in dict(req.fields()).items():
            if not key in deleteValues:
                self.middleKitObjectEval(self.admin,key,value,type = "Boolean")
        
        store.saveChanges()
        self.writeBody()

    def backPage(self):
        return "dcs_admin"

    def localTitle(self):
        return "Administration"

    def writeLocalContent(self):
        admin_bool = { "usePrefix": False ,"useSuffix": False,"newAssemblies": False,"newOperations": False,"newJobs": False  }
        
        prefix = self.escape(self.admin.prefix())
        suffix = self.escape(self.admin.suffix())

        for key,value in admin_bool.items():
            if bool( getattr(self.admin,"_%s" % key) ): admin_bool[key]="CHECKED"
            else: admin_bool[key]=""

        admin_bool["userJobs"] = ""
        admin_bool["autoJobs"] = ""

        if self.admin.jobNumbering() == "user":
            admin_bool["userJobs"] = "CHECKED"
        else:
            admin_bool["autoJobs"] = "CHECKED"
                
        last_used = int(LastUsedFactory().getNextJobNumber() - 1) #Last and Next jobs numbers
        next_used = last_used + 1
    
        self.redirectIfTrue( UserFactory().redirectIfAccessDenied( self.session().value("userLoggedOn"), "admin_globals","dcs_admin/user_preferences" ) ) 

        self.writeln("""<table width="100%">""")
        self.writeln("""<tr>\n\t<td>""")
       
        self.writeln( self.subTabs() )

        self.writeln("""\t</td>\n</tr>\n<tr>\n\t<td>""")
        self.writeln("""<table width="50%" align="center">""")
        
        self.writeln("<form name=admin_form method=post action='./index?_action_=change'>")
            
        self.writeln("<tr class='table_header'>")
        self.writeln("\t<th align='center' colspan='3'>Job Numbers</th>")
        self.writeln("</tr>")

        #Allow creation of new jobs     
        self.writeln("<tr>")
        self.writeln("""\t<td COLSPAN=3 ALIGN=left CLASS=body_desc><INPUT TYPE=checkbox NAME=NewJobs CLASS=body_data value=1 %s onClick="set_job_creation();">&nbsp;Allow creation of new Jobs&nbsp;</td></tr>""" % (admin_bool["newJobs"]) )
        
        #Checkbox Name userJobs
        self.writeln("<tr>")
        self.writeln("""\t<td COLSPAN=3 ALIGN=left CLASS=body_desc><INPUT TYPE=checkbox NAME=JobNumbering ID=userJobs CLASS=body_data value='user' %s onClick="set_force_user();">&nbsp;Force user defined job numbers&nbsp;</td></tr>""" % admin_bool["userJobs"])

        #Checkbox Name autoJobs
        self.writeln("<tr>")
        self.writeln("""\t<td COLSPAN=3 ALIGN=left CLASS=body_desc><INPUT TYPE=checkbox NAME=JobNumbering ID=autoJobs CLASS=body_data value='auto' %s onClick="set_force_auto();">&nbsp;Force automatic job numbers&nbsp;</td></tr>""" % (admin_bool["autoJobs"]) )
    
        #Checkbox Name last_used
        self.writeln("<tr>")
        self.writeln("""\t<td ALIGN=left CLASS=body_desc COLSPAN=2>Last Used #:&nbsp;</td><td align=left><INPUT TYPE=text NAME=last_used CLASS=body_data SIZE=20 MAXLENGTH=20 value="%s" onClick="validate_last_used(this.value);">&nbsp;</td>""" % last_used);
        self.writeln("</tr>")

        #Checkbox Name usePrefix
        self.writeln("<tr>")
        self.writeln("""\t<td ALIGN=left CLASS=body_desc><INPUT TYPE=checkbox NAME=UsePrefix CLASS=body_data value=1 %s onClick="set_next_available();">&nbsp;Use Prefix:&nbsp;</td>\n\t<td colspan=2 align=left><INPUT TYPE=text NAME=Prefix CLASS=body_data SIZE=5 MAXLENGTH=5 value="%s" onClick="set_next_available();">&nbsp;</td></tr>""" % (admin_bool["usePrefix"],prefix) )
        
        #Checkbox Name useSuffix
        self.writeln("<tr>")
        self.writeln("""\t<td ALIGN=left CLASS=body_desc><INPUT TYPE=checkbox NAME=UseSuffix CLASS=body_data value=1 %s onClick="set_next_available();">&nbsp;Use Suffix:&nbsp;</td>\n\t<td colspan=2 align=left><INPUT TYPE=text NAME=Suffix CLASS=body_data SIZE=5 MAXLENGTH=5 value="%s" onClick="set_next_available();">&nbsp;</td></tr>""" % (admin_bool["useSuffix"],suffix) )

        # Next Job Number
        self.writeln("<tr CLASS=table_header_gray><th colspan='3'>Next Default Job Number</th></tr>")
        self.writeln("<tr><td colspan='3' align=center><input type=text name=nextfield class=title style='border:none; text-align: center;' size=35 value='%d' READONLY></td></tr>" % next_used)

        # Job Operations header
        self.writeln("<tr class='table_header'>")
        self.writeln("\t<th align='center' colspan='3'>Job Operations</th>")
        self.writeln("</tr>")
        
        #Checkbox Name newOperations
        self.writeln("<tr>")
        self.writeln("\t<td ALIGN=left CLASS=body_desc><INPUT TYPE='checkbox' NAME='NewOperations' CLASS='body_desc' value='1' %s>&nbsp;Allow creation of new Operations&nbsp;</td></tr>" % admin_bool["newOperations"])
        
        self.writeln("<tr class='table_header'>")
        self.writeln("\t<th align='center' colspan='3'>Assemblies</th>")
        self.writeln("</tr>")

        #Checkbox Name newAssemblies
        self.writeln("<tr>")
        self.writeln("\t<td ALIGN=left CLASS=body_desc><INPUT TYPE='checkbox' NAME='NewAssemblies' CLASS='body_desc' value='1' %s>&nbsp;Allow creation of new Assemblies&nbsp;</td></tr>" % admin_bool["newAssemblies"])

        self.writeln("""<tr>
        <td COLSPAN=3 ALIGN=left><p ALIGN=center><INPUT TYPE=image SRC="../images/submit1.gif" onMouseDown="this.src='../images/submit1down.gif';" onMouseOut="this.src='../images/submit1.gif';"></p></td>
        </tr>""")
        
        self.writeln("</FORM>")
        # FORM END
        self.writeln("</table>")
        self.writeln("""\t</td>\n</tr>\n</table>""")

        self.writeln("<script>last_used=document.admin_form.last_used.value;set_all();</script>")
