# vim: ts=4 et sts=4 sw=4 autoindent

from etc import siteindex
from sndcs.JobHedFactory import JobHedFactory
from sndcs.EmployeeFactory import EmployeeFactory
from lib.SecureSitePage import SecureSitePage
from index import index
from lib.Reporting import Reporting
from sndcs.UserFactory import UserFactory

class reports(index,SecureSitePage,Reporting):

    def __init__(self):
        SecureSitePage.__init__(self, siteindex["dcs_admin"], 6,False, SecureSitePage.mainScripts + [ "../js/jscalendar/calendar.js", "../js/jscalendar/calendar-en.js" ])

    def localTitle(self):
        return "Reporting"

    def actions(self):
        return index.actions(self) + [ "reports" ]

    def reports(self):
        req = self.request()
        self.redirectIfTrue( UserFactory().redirectIfAccessDenied( self.session().value("userLoggedOn"), "admin_globals" ) ) 

        printable = req.hasField("printable")
        
        if printable:
            self.message = ""
        else:
            self.message = "<body bgcolor='#ffffff'>"

        self.message += ("""<table width=100%>""")
        if not printable:
            self.message += ( self.subTabs() )
        self.message += ("""<tr align=center>\n\t<td>""")

        if req.hasField("employeeReport"):
            employeeObjId = self.returnType(req.field("employeeSerialNum",""),int,None)
            startStamp = self.convertToDateTime(date = req.field("startStamp",""))
            endStamp = self.convertToDateTime(date = req.field("endStamp",""))
            if employeeObjId:
                self.message += ( """<table width="75%%">\n%s\n</table>""" % self.getEmployeeReport( employeeObjId, startStamp, endStamp )[0] )
            else:
                self.message += ( """<table width="75%%">\n%s\n</table>""" % self.getGlobalEmployeeReport( startStamp, endStamp ) )
        elif req.hasField("jobHedReport"):
            jobHedObjId = self.returnType(req.field("jobHedSerialNum",""),int,None)
            self.message += ( self.getJobHedReport( jobHedObjId ) )
        else:
            self.goToPage( self.backPage() )

        if not printable:
            self.message += ("""<A HREF="./reports"><IMAGE SRC="../images/back1.gif" onMouseDown="this.src='../images/back1down.gif';" onMouseOut="this.src='../images/back1.gif';" BORDER=0></A>\n\t</td>\n\t</tr>\n</table>""")
            self.writeBody() 
        else:
            self.message += ("""\t</td>\n</tr>\n</table>\n</body>""")
            self.writeln(self.message)
            self.message = ""

    def writeStyleSheet(self):
        req=self.request()
        if req.hasField("printable"):
            return """ """
        else:
            self.writeln("""<link rel="stylesheet" type="text/css" media="all" href="../css/calendar-blue.css" title="blue" />""")
            index.writeStyleSheet(self)

    def backPage(self):
        req=self.request()
        if req.hasField("jobHedReport"):
            return "dcs_admin/reports?jobHedReport=1&jobHedSerialNum=%s&_action_=reports" % req.field("jobHedSerialNum",0)
        elif req.hasField("employeeReport"):
            return "dcs_admin/reports?employeeReport=1&employeeSerialNum=%s&_action_=reports&startStamp=%s&endStamp=%s" % ( req.field("employeeSerialNum",0),req.field("startStamp",0),req.field("endStamp",0) )
        else:
            return "dcs_admin/reports"

    def newTab(self):
        if not self.backPage() == "dcs_admin/reports":
            return [ "Printable Version", "../" + self.backPage() +"&printable=1" ]
        else:
            return index.newTab(self)

    def writeLocalContent(self):
        self.redirectIfTrue( UserFactory().redirectIfAccessDenied( self.session().value("userLoggedOn"), "admin_globals" ) ) 

        if self.message == "":
            self.writeln("""<table width=100%>""")
            self.writeln( self.subTabs() )
            
            self.writeln("""<tr>\n\t<td>""")
            self.writeln("""<table width="75%" align="center">""")
            
            self.writeln("""<form method=post action='./reports?_action_=reports&employeeReport=1'>""")

            self.writeln("""<tr class=table_header>\n\t<th colspan=4>""")
            self.writeln("""Employee Reports""")
            self.writeln("""\t</th>\n</tr>""")

            self.writeln("""<tr align=center>\n\t<td colspan=4>\n\t\t<select name=employeeSerialNum>\n<option name="ALL">ALL</option>\n""")
            
            for employee in EmployeeFactory().getEmployees():
                self.writeln( self.write_option(employee.serialNum(), "%s %s" % (employee.empId(), employee.properName()) ) )

            self.writeln("""\n\t\t</select>\n\t</td>\n</tr>""")
                
            self.writeln("""<tr><td class=body_desc align=right colspan=2>Start Stamp: <input type='text' size=10 maxlength=10 ID='start_stamp' NAME='startStamp'>&nbsp;<img src=../images/button1.gif class=pointer ID='start_trigger'></td><td class=body_desc align=left colspan=1>End Stamp: <input type='text' size=10 maxlength=10 ID='end_stamp' NAME='endStamp'>&nbsp;<img class=pointer src=../images/button1.gif ID='end_trigger'></td></tr>""")    
           
            self.writeln("""<script language="javascript" src="../js/jscalendar/calendar-setup.js"></script>""")

            self.writeln("""\n<script type="text/javascript">
             Calendar.setup({
                                inputField     :    "start_stamp",     // id of the input field
                                ifFormat       :    "%m/%d/%Y",      // format of the input field
                                button         :    "start_trigger",  // trigger for the calendar (button ID)
                                align          :    "Tr",           // alignment (defaults to "Bl")                                       
                                singleClick    :    true
                });
             Calendar.setup({
                                inputField     :    "end_stamp",     // id of the input field
                                ifFormat       :    "%m/%d/%Y",      // format of the input field
                                button         :    "end_trigger",  // trigger for the calendar (button ID)
                                align          :    "Tr",           // alignment (defaults to "Bl")                                       
                                singleClick    :    true
                });

            </script>""")

           
            self.writeln("""<tr align=center>\n\t<td colspan=4>\n\t\t<INPUT TYPE=image SRC="../images/search1.gif" onMouseDown="this.src='../images/search1down.gif';" onMouseOut="this.src='../images/search1.gif';">\n\t</td>\n</tr>""")

            self.writeln("""</form>""")

            self.writeln("""<form method=post action='./reports?_action_=reports&jobHedReport=1'>""")
            
            self.writeln("""<tr class=table_header>\n\t<th colspan=4>""")
            self.writeln("""JobHed Reports""")
            self.writeln("""\t</th>\n</tr>""")
            
            self.writeln("""<tr align=center>\n\t<td colspan=4>\n\t\t<select name=jobHedSerialNum>""")

            for jobhed in JobHedFactory().getJobHeds():
                self.writeln( self.write_option( jobhed.serialNum(), "%s - %s - %s" %  ( jobhed.number(), jobhed.customer().name(), jobhed.itemDescription() ) ) )

            self.writeln("""\n\t\t</select>\n\t</td>\n</tr>""")

            
            self.writeln("""<tr align=center>\n\t<td colspan=4>\n\t\t<INPUT TYPE=image SRC="../images/search1.gif" onMouseDown="this.src='../images/search1down.gif';" onMouseOut="this.src='../images/search1.gif';">\n\t</td>\n</tr>""")

            self.writeln("""</form>""")

            self.writeln("""</table>""")
            self.writeln("""\n\t</td>\n</tr>""")
            self.writeln("""</table>""")
        else:
            self.writeln( self.message )
