from lib.SecureSitePage import SecureSitePage
from index import index
from etc import siteindex

from sndcs.LastUsedFactory import LastUsedFactory
from sndcs.AdminGlobalFactory import AdminGlobalFactory
from sndcs.UserFactory import UserFactory
from sndcs.Store import store
from sndcs.DepartmentFactory import DepartmentFactory
from sndcs.Department import Department
from sndcs.TerminalFactory import TerminalFactory
from sndcs.Terminal import Terminal

class terminals(index,SecureSitePage):
    
    def __init__(self):
        SecureSitePage.__init__(self,siteindex["dcs_admin"],4,False, SecureSitePage.mainScripts )

    def actions(self):
        return SecureSitePage.actions(self) + [ "new_terminal", "edit_terminals", "delete_terminal" ]

    def backPage(self):
        return "dcs_admin/terminals"

    def localTitle(self):
        return "Edit Terminal Settings"

    def new_terminal(self):
        self.redirectIfTrue( UserFactory().redirectIfAccessDenied( self.session().value("userLoggedOn"), "admin_globals" ) ) 
        
        store.addObject( TerminalFactory().new() )
        store.saveChanges()

        self.goToPage(self.backPage())

    def edit_terminals(self):
        req = self.request()
        serialNum = self.returnType(req.field("serialNum",""),int,None)

        self.redirectIfTrue( UserFactory().redirectIfAccessDenied( self.session().value("userLoggedOn"), "admin_globals" ) ) 
        
        for terminal in TerminalFactory().getTerminals():
            if req.hasField("terminal%s" % terminal.serialNum() ):
                    terminalName = self.escape(req.field("terminal%s" % terminal.serialNum(),"").replace("'",""))
                    if not terminalName == "" and not store.fetchObjectsOfClass(Terminal,"""WHERE terminalName='%s' AND serialNum<>%s""" % (terminalName,terminal.serialNum() ) ):
                        terminal.setTerminalName( terminalName )
            for department in DepartmentFactory().getDepartments():
                if req.hasField("%sdepartment%s" % (department.code(),terminal.serialNum()) ):
                        if not store.fetchObjectsOfClass(Department,"WHERE terminalObjId=%s AND code='%s'"% (terminal.serialNum(),department.code()) ):
                            tmpdept = DepartmentFactory().new()
                            tmpdept.setCode( department.code() )
                            tmpdept.setDescription( department.description() )
                            terminal.add(department)
                            terminal.addToDepartment(tmpdept)
                else:
                        tmpdelete = store.fetchObjectsOfClass(Department,"WHERE terminalObjId=%s AND code='%s'" % ( terminal.serialNum(),department.code() ) )
                        if tmpdelete:
                            store.deleteObject(tmpdelete[0])
                            terminal.delete(tmpdelete[0])
            store.saveChanges()
        self.goToPage(self.backPage() )

    def delete_terminal(self):
        req = self.request()
        serialNum = req.field("serialNum","")
        if self.isType(serialNum,int):
            self.redirectIfTrue( UserFactory().redirectIfAccessDenied( self.session().value("userLoggedOn"), "admin_globals" ) )     
            
            terminal = TerminalFactory().getTerminalBySerialNum( int(serialNum) )
            for department in terminal.department():
                #terminal.delete(department)
                store.deleteObject(department)
                store.saveChanges()    
            
            if not terminal.referencingObjectsAndAttrs():
                terminal.clearSettings()
                store.deleteObject(terminal)
            
            store.saveChanges()
        self.goToPage(self.backPage())

    def writeLocalContent(self):
        self.writeln("""<table width="100%">""")
        self.writeln( self.subTabs() )
        
        allActiveDepartments = DepartmentFactory().getDepartments()

        self.writeln("""<tr>\n\t<td>""")
        self.writeln("""<table width="100%">""")
        self.writeln("""<form action='./terminals?_action_=edit_terminals' method='post'>""")
        self.writeln("""<tr class=table_header>\n\t<th colspan=%s>Terminal Settings</th>\n</tr>""" % ( len(allActiveDepartments) + 2 ) )
        
        self.writeln("""<tr class=body_desc align=left><td>Terminal ID</td>""")
        
        for department in allActiveDepartments:
            self.writeln("""<td width="10%%" align=center>&nbsp;%s&nbsp;</td>""" % ("%s %s" % (department.code(),department.description() ) ) )
      
        self.writeln("""<td align=center width="10%%">&nbsp;Delete&nbsp;</td></tr>""")

        for terminal in TerminalFactory().getTerminals():
            self.writeln("""<tr>\n\t<td align=left><input type='text' name='terminal%s' value="%s" size=30></td>""" % ( terminal.serialNum(),self.escape( terminal.terminalName() ) ) )
            
            for testDepartment in allActiveDepartments:
                testChecked = self.ifTrueReturn(store.fetchObjectsOfClass(Department,"WHERE terminalObjId=%s AND code='%s'" % (terminal.serialNum(),testDepartment.code() ) ),"CHECKED","")

                self.writeln("""<td align=center><input type='checkbox' name='%sdepartment%s' value=1 %s></td>""" % (testDepartment.code(),terminal.serialNum(),testChecked) )

            
            self.writeln("""<td align=center>\n<a href='./terminals?_action_=delete_terminal&serialNum=%s'><img src='../images/button3.gif' border=0></a>\n\t</td>\n</tr>""" % (terminal.serialNum()) )

        self.writeln("""<tr height='40px' align=center><td colspan='%s'><INPUT TYPE=image SRC="../images/submit1.gif" onMouseDown="this.src='../images/submit1down.gif';" onMouseOut="this.src='../images/submit1.gif';">&nbsp;<A HREF="./terminals?_action_=new_terminal"><IMG SRC="../images/new.gif" onMouseDown="this.src='../images/newdown.gif';" onMouseOut="this.src='../images/new.gif';" style="border-style: none;"></A></td></tr>""" % (len(allActiveDepartments) + 2) ) 

        self.writeln("""\t</td>\n</tr>""")
        self.writeln("""</form>\n</table>""")
        self.writeln("""\t</td>\n</tr>\n</table>""")
