# vim: ts=4 et sts=4 sw=4 autoindent

from lib.SecureSitePage import SecureSitePage
from index import index

from sndcs.Customer import Customer
from sndcs.CustomerFactory import CustomerFactory
from sndcs.UserFactory import UserFactory
from sndcs.StateFactory import StateFactory
from etc import siteindex
from sndcs.Store import store

class customers(index,SecureSitePage):

    def actions(self):
            return SecureSitePage.actions(self) + [ "edit", "delete" ]

    def newTab(self):
        req=self.request()
        serialNum=req.field("serialNum",False)
        if serialNum and str(self.session().value("backPage","")).count("search"):
            return ["Back To Search Results",str(self.session().value("backPage")).replace("dcs_edit_customers/","")]
        else:
            return index.newTab(self)

    def backPage(self):
        if str(self.session().value("backPage","")).count("search"):
            return self.session().value("backPage")
        else:
            return index.backPage(self)

    def writeJavascript(self):
            return """<script language='javascript' type='text/javascript'>
            <!--
            function validOnSubmit(myForm) {
                if ( myForm.CustomerId.value == "" ) {
                    alert("Error: Please enter a Customer Id");
                    myForm.CustomerId.focus();
                    return false;
                }
                
                if ( myForm.Name.value == "" ) {
                    alert("Error: Please enter a Customer Name");
                    myForm.Name.focus();
                    return false;
                }

                /*if ( myForm.Address.value == "" ) {
                    alert("Error: Please enter a Address");
                    myForm.Address.focus();
                    return false;
                }

                if ( myForm.City.value == "" ) {
                    alert("Error: Please enter a city");
                    myForm.City.focus();
                    return false;
                }
               
                if ( ( myForm.ZipCode.value=="" ) || ( isInteger(myForm.ZipCode.value) == false )  || (myForm.ZipCode.value.length != 5) ) {
                    alert("Error: Please enter a valid Zip Code");
                    myForm.ZipCode.focus();
                    return false;
                }*/
                return true;    
            };
            //-->
            </script>"""


    def edit(self):
            req = self.request()
            deleteValues = [ "x", "y", "_action_" ]
            saverequest = req.fields()
            serialNum = self.returnType(req.field("serialNum",""),int,0)

            if serialNum:
                self.redirectIfTrue( UserFactory().redirectIfAccessDenied( self.session().value("userLoggedOn"), "edit_customers" ) ) 
                tmpedit = CustomerFactory().getCustomerBySerialNum(serialNum)
                deleteValues.append("serialNum")
            else:
                self.redirectIfTrue( UserFactory().redirectIfAccessDenied( self.session().value("userLoggedOn"), "create_customers" ) ) 
                tmpedit = CustomerFactory().new()

            if store.fetchObjectsOfClass(Customer,clauses="WHERE CustomerId='%s' AND serialNum<>%s" % ( saverequest["CustomerId"],serialNum ) ):
                store.saveChanges()
                self.message = ("""<b>Error:</b> A Customer already exists with that Customer ID""")
                self.writeBody()
                return

            if req.hasField("State"):
                state = StateFactory().getStateByAbbev(req.field("State") )
                deleteValues.append("State")
                tmpedit.setState(state)

            for key,value in saverequest.items():
                if not key in deleteValues:
                    self.middleKitObjectEval(tmpedit,key,value)

            store.addObject(tmpedit)
            store.saveChanges()
            self.goToPage(self.backPage())
            
    def delete(self):
            req = self.request()
            serialNum = self.returnType(req.field("serialNum",""),int,None)
            
            self.redirectIfTrue( UserFactory().redirectIfAccessDenied( self.session().value("userLoggedOn"), "delete_customers" ) ) 
            
            if self.isType(serialNum,int):
                tmpdelete = CustomerFactory().getCustomerBySerialNum(serialNum)
                if not tmpdelete.referencingObjectsAndAttrs():
                    store.deleteObject(tmpdelete)
                else:
                    tmpdelete.setStatus("Blocked")

            store.saveChanges()    
            self.goToPage("dcs_edit_customers")

    def localTitle(self):
            req = self.request()
            if req.hasField("serialNum"):
                return "Edit Customer"
            else:
                return "New Customer"


    def writeLocalContent(self):
            req=self.request()
            
            self.writeln("""<table width="100%">""")

            self.writeln("""<tr>\n\t<td>""")
            self.writeln( self.subTabs() )
            self.writeln("""\n\t</td>\n</tr>""")

            self.writeln("""<tr>\n\t<td>""")

            self.writeln("""<table align=center width="50%">""")

            if self.message=="":


                serialNum,recordType,customerId,name,address1,address2,city,state,zipCode = [ False,"Create New","","","","","","","" ]
                
                status = [ "Selected","","" ]
                 
                serialNum = self.returnType(req.field("serialNum",""),int,None)
                if serialNum:
                        tmpread = CustomerFactory().getCustomerBySerialNum(serialNum)
                        
                        if tmpread.state():
                            state = tmpread.state().abbreviation()
                        
                        if tmpread.status()=="Active":
                                status = [ "SELECTED","","" ]
                        elif tmpread.status()=="Prospect":
                                status = [ "","SELECTED","" ]
                        else:
                                status = [ "","","SELECTED" ]
                        
                        recordType,customerId,name,address1,address2,city,zipCode = [ "Edit ",tmpread.customerId(), tmpread.name(), tmpread.address(), tmpread.address2(), tmpread.city(),tmpread.zipCode() ]
                        

                self.writeln( self.writeJavascript() )

                self.writeln("""<form action='./customers?_action_=edit' method='post' onSubmit='return validOnSubmit(this);'>
                    <tr><th class='table_header' colspan=4> %s Customer Record<th><tr>
                <tr><td class=body_desc align=left>Customer ID:&nbsp;</td><td><input type='text' name='CustomerId' value='%s'></td><td class=body_desc align=left>Customer Name:&nbsp;</td><td><input type='text' name='Name' value='%s'></td>
                
                <tr><td colspan=4 align=center><hr></td></tr>
                
                <tr><td class=body_desc align=left>Address:&nbsp;</td><td><input type='text' name='Address' value='%s'></td><td class=body_desc align=left>Status:&nbsp;</td><td>
                    <select name="Status">
                        <option value="Active" %s>Active
                        <option value="Prospect" %s>Prospect
                        <option value="Blocked" %s>Blocked
                    </select>
                </td></tr>


                <tr><td class=body_desc align=left>City:&nbsp;</td><td><input type='text' name='City' value='%s'></td>""" % (recordType,self.escape( customerId ),self.escape( name ),self.escape( address1 ),status[0],status[1],status[2],self.escape( city ) ) )
                
                
                self.writeln("<td class=body_desc align=left>State:&nbsp;</td><td><select name='State'>")
            

                self.writeln( self.state_option(state) )
            
                self.writeln("""</select>
                </td></tr>

                <tr><td class=body_desc align=left>Zip Code:&nbsp;</td><td colspan=3 align=left><input type='text' name='ZipCode' value='%s' MAXLENGTH=5 SIZE=5></td></tr>
            
                <tr height='40px'><td colspan=4 align=center><input type=image src="../images/submit1.gif" onMouseDown="this.src='../images/submit1down.gif';" onMouseOut="this.src='../images/submit1.gif';">&nbsp;<A HREF="./index.py"><IMAGE SRC="../images/back1.gif" onMouseDown="this.src='../images/back1down.gif';" onMouseOut="this.src='../images/back1.gif';" BORDER=0></A></td></tr>""" % self.escape(zipCode) )


                if serialNum: self.writeln("<input type='hidden' name='serialNum' value='%s'>\n</form>" % serialNum)
            else:
                self.writeln( """<tr align=center><td><font color="#ff0000">%s</font></td></tr>""" % self.message )

            self.writeln("""</table>\n\t</td>\n</tr>""")
            self.writeln("""</table>""")
