# vim: ts=4 et sts=4 sw=4 autoindent

from lib.SecureSitePage import SecureSitePage
from etc import siteindex

from sndcs.OperationFactory import OperationFactory
from sndcs.EmployeeFactory import EmployeeFactory
from sndcs.DepartmentFactory import DepartmentFactory
from sndcs.LaborHedFactory import LaborHedFactory
from sndcs.UserFactory import UserFactory
from sndcs.Store import store

import datetime

class index(SecureSitePage):

    header = ("""\t<tr CLASS=table_header>
                <th ALIGN=left>&nbsp;Emp #</th>
                <th ALIGN=left>Last Name</th>
                <th ALIGN=left>First Name</th>
                <th ALIGN=left>Status</th>
                <th ALIGN=center>Shift</th>
                <th ALIGN=center>Oprs.</th>
                <th ALIGN=center>Photo</th>
                <th ALIGN=center>Edit</th>
                <th ALIGN=center>Delete</th>
            </tr>""")
    
    def __init__(self,subtab = 1):
        SecureSitePage.__init__(self,siteindex["dcs_edit_employees"],subtab,False,[ "../js/sndcs2_main.js","../js/jscalendar/calendar.js", "../js/jscalendar/calendar-en.js" ])

    def actions(self):
        return SecureSitePage.actions(self) + ["delete"]
        
    def writeStyleSheet(self):
        SecureSitePage.writeStyleSheet(self)
        self.writeln("""<link rel="stylesheet" type="text/css" media="all" href="../css/calendar-blue.css" title="blue" />""")

    def delete(self):
        req = self.request()
        serialNum = self.returnType(req.field("serialNum",""),int,None)

        if serialNum:
            self.redirectIfTrue( UserFactory().redirectIfAccessDenied( self.session().value("userLoggedOn"), "delete_employee" ) ) 
            deleteemployee = EmployeeFactory().getEmployeeBySerialNum(serialNum)
                
            if not deleteemployee.referencingObjectsAndAttrs(): # Delete Employee if there is no laborHeds otherwise change to Inactive
                deleteemployee.setStatus("Terminated")
                deleteemployee.setTermDate( datetime.date.today() )
                store.addObject(deleteemployee)
                store.saveChanges()
            else:
                tmpphoto = deleteemployee.picture()
                for department in EmployeeFactory().getDepartmentsOfEmployeeBySerialNum(deleteemployee.serialNum()):
                    store.deleteObject(department)
                    store.saveChanges()

                store.deleteObject(deleteemployee)
                deleteemployee.delete() # Send out PYRO event
                store.saveChanges()
            
                if tmpphoto:
                    store.deleteObject(tmpphoto)
                    store.saveChanges()
    
            self.goToPage("dcs_edit_employees")
    
    def newTab(self):
        return ["New Employee","edit_employee"]

    def localTitle(self):
        return "Employees"

    def backPage(self):
        return self.siteRoot() + "dcs_edit_employees"

    def subTabs(self):
        subtab1 = self.write_div(id="subtab1",classname="subtab",text="Main", construct = [ 1, siteindex["dcs_edit_employees"], "index" ]);
        subtab2 = self.write_div(id="subtab2",classname="subtab",text="Search", construct = [ 2, siteindex["dcs_edit_employees"], "search" ]);
        return """<tr><td><table align="left"><tr><td>%s</td><td>%s</td></tr></table></td></tr>""" % (subtab1,subtab2)

    def writeLocalContent(self):

        self.writeln("""<table width="100%">""")
        
        self.writeln( self.subTabs() )

        self.writeln("""<tr>\n\t<td>""")
        self.writeln("""<table width="100%">""")

        self.writeln( self.header )

        self.writeln("""<tr bgcolor='#acaaac' class=list_data><td colspan=9 align=left><b>&nbsp;Department:&nbsp;</b>Misc.</td></tr>""")
        self.start_color()
       
        def employeeRow(employee):
                bgcolor = self.next_color()

                shiftDescription = employee.shift()
                if shiftDescription:
                    shiftDescription = shiftDescription.name()

                return """
                <tr bgcolor='%s'>
                        <td CLASS=list_data WIDTH='50px'>%s<td CLASS=list_data>%s</td>
                        <td CLASS=list_data>%s</td><td ALIGN='left' CLASS='list_data'>%s</td><td ALIGN='center' CLASS=list_data>%s</td>
                        <td ALIGN='center' WIDTH='30px'><a href='./task_list?employeeSerialNum=%s'><img src='../images/button2.gif' border=0></td>
                        <td ALIGN='center' WIDTH='30px'><a href='./photo?empId=%s'><img src='../images/button1.gif' border=0></a></td>
                        <td ALIGN='center' WIDTH='30px'><a href='./edit_employee?serialNum=%s'><img src='../images/button4.gif' border=0></td>
                        <td ALIGN='center' WIDTH='30px'><a href="./index?_action_=delete&serialNum=%s"><img src='../images/button3.gif' border=0></a></td>
                </tr>""" % (bgcolor,employee.empId(),self.escape( employee.lastName() ),self.escape( employee.firstName() ),employee.status(),self.escape(shiftDescription),employee.serialNum(),employee.empId(),employee.serialNum(),employee.serialNum() )


        for employee in EmployeeFactory().getEmployees():
            if not EmployeeFactory().getDepartmentsOfEmployeeBySerialNum( employee.serialNum() ):
                self.writeln( employeeRow(employee) )

        for department in DepartmentFactory().getDepartments():
            self.start_color()
            self.writeln("""<tr bgcolor='#acaaac' class=list_data><td align=left colspan=9><b>&nbsp;Department:&nbsp;</b>%s - %s <a href='../dcs_edit_departments/task_list?serialNum=%s'><img src='../images/button1.gif' border=0></a>&nbsp;</td></tr>""" % (department.code(),department.description(),department.serialNum()) )
            for employee in EmployeeFactory().getEmployeesOfDepartmentCode(department.code() ):
                self.writeln( employeeRow(employee.employee()) )

        self.writeln( self.header )
        self.writeln("""
        <tr HEIGHT='60px'>
            <td COLSPAN=9 ALIGN=center><A HREF="./edit_employee"><IMG SRC="../images/new.gif" onMouseDown="this.src='../images/newdown.gif';" onMouseOut="this.src='../images/new.gif';" style="border-style: none;"></A></td>
        </tr>
        </table>
        
            </td>
        </tr>
        </table>""")
