# vim: ts=4 et sts=4 sw=4 autoindent

from lib.SecureSitePage import SecureSitePage
from index import index
from etc import siteindex

from sndcs.Store import store
from sndcs.BOMFactory import BOMFactory
from sndcs.ItemFactory import ItemFactory

class bom(index,SecureSitePage):
    def __init__(self,subtab=1):
        SecureSitePage.__init__(self,siteindex["dcs_edit_items"],subtab,clearSearchResults=False)

    def localTitle(self):
        req=self.request()
        if req.hasField("serialNum"):
            return "Edit BOM"
        else:
            return "Create New BOM"


    def newTab(self):
        req=self.request()
        serialNum=req.field("serialNum",False)
    
        if serialNum:
            return ["New BOM","bom?new=1&serialNum=%s" % serialNum]
        else:
            return index.newTab(self)

    def writeJavascript(self):
        return """
        <script language='javascript' type='text/javascript'>
        function checkIfOpen(MyCheckbox,Number,Description) {
            if (MyCheckbox.checked) {
                if (Number) {
                    Number.disabled=false;
                }
                Description.disabled=false;
            }
            else {
                if (Number) {
                    Number.disabled=true;
                }
                Description.disabled=true;
            }
        }
        </script>"""

    def displayBOMItems(self,tmpbomlist,bomAlreadyList=[ [],{} ]):
        try:
            tmpbom = tmpbomlist[0]
        except TypeError:
            tmpbom = tmpbomlist

        serialNum = tmpbom.serialNum()
        itemSerialNum = tmpbom.material().serialNum()
        if not bomAlreadyList[0].count(itemSerialNum):
            bomAlreadyList[0].append(itemSerialNum)
            output = """<td class=list_data align=left>&nbsp;%s</td>
                        <td class=list_data align=left><input disabled type='text' name='Description%s' value='%s' size=100 maxlength=100></td>""" % (tmpbom.number(),itemSerialNum,tmpbom.material().description() ) # Size is bigger than the maxlength because it doesn't render pretty otherwise.
            
            
            output += """<td align=center><input type='checkbox' name='editItem%s' value=1 onClick='checkIfOpen(this,0,this.form.Description%s);'></td>
                         <td align=center><a href='./routing?serialNum=%s&referrentBOM=%s'><img src='../images/button1.gif' border=0></td><td align=center><a href='./bom?_action_=delete&delete_type=bom&serialNum=%s&referrentBOM=%s'><img src='../images/button3.gif' border=0></a></td>""" % (itemSerialNum,itemSerialNum,itemSerialNum,serialNum,serialNum,self.request().field("serialNum")) 
               
            bomAlreadyList[1][tmpbom.number()] = output

        for bomitem in tmpbom.material().bom():
            materialObj = bomitem.material().serialNum()
            itemObj = bomitem.item().serialNum()
            if not bomAlreadyList[0].count(serialNum):
                bomAlreadyList[0].append(serialNum)
                self.displayBOMItems( bomitem,bomAlreadyList )

        return bomAlreadyList

    def backPage(self):
        req = self.request()
        if req.field("serialNum",False):
            return "dcs_edit_items/bom?serialNum=%s" % req.field("serialNum")
        elif req.has_field("referrentBOM"):
            return "dcs_edit_items/bom?serialNum=%s" % req.field("referrentBOM")
        else:
            return "dcs_edit_items"
        
    def localTitle(self):
        return "Edit Bill of Material"

    def writeLocalContent(self):
        req = self.request()
        serialNum = self.returnType(req.field("serialNum",""),int,0)
        
        if not serialNum:
            self.goToPage("dcs_edit_items")
            return


        self.writeln("""<table width="100%">""")
        self.writeln("""<tr>\n\t<td>""")

        self.writeln( self.subTabs() )
        self.writeln("""\t</td>\n</tr>\n<tr>\n\t<td>""")

        if req.hasField("new"):
                self.writeln("""<table width="50%" align="center">""")
                self.writeln("""
                <form action='./bom' method='post' onSubmit='return validOnSubmit(this);'>
                <tr class=table_header><th align=center colspan=2>Add New BOM</th></tr>
                <tr><td class=body_desc>&nbsp;BOM Item:&nbsp;
                <select name=BomItem>""")
        
                for item in ItemFactory().getItems():
                    if not int(serialNum)==item.serialNum():
                        self.writeln( self.write_option(item.serialNum(),( "%s - %s" % (item.number(),item.description() ) ) ) )

                self.writeln("""
                </select>
                </td></tr>
                <tr><td colspan=2 align=center><INPUT TYPE=image SRC="../images/submit1.gif" onMouseDown="this.src='../images/submit1down.gif';" onMouseOut="this.src='../images/submit1.gif';">&nbsp;<A HREF="./bom?serialNum=%s"><IMAGE SRC="../images/back1.gif" onMouseDown="this.src='../images/back1down.gif';" onMouseOut="this.src='../images/back1.gif';" BORDER=0></A></td></tr>
                <input type='hidden' name='_action_' value='edit'>
                <input type='hidden' name='edit_type' value='bom'>
                <input type='hidden' name='serialNum' value='%s'>

                </form>
                </table>""" % (serialNum,serialNum) )
        else:        
        
            self.writeln( self.writeJavascript() )

            self.writeln("""<table width="100%">""")
            
            self.writeln("""<form action='./bom'  method='post'>
            <tr class=table_header><th colspan=5>BOM's</th></tr>
            <tr class=table_header><th align=left width='10%'>&nbsp;Number&nbsp;</th><th align=left>Description</th><th width='30px'>Edit</th><th width='30px'>&nbsp;Rot.</th><th width='30px'>&nbsp;Delete</th></tr>""")
            
            output = {}
            self.start_color()
            for tmpbom in BOMFactory().getBOMsByItemSerialNum( int( serialNum ) ):
                output.update(self.displayBOMItems(tmpbom,[ [],{} ])[1])
           
            keys = output.keys()
            keys.sort()

            for key in keys:
                bgcolor = self.next_color()
                self.writeln( """<tr bgcolor='%s'""" % bgcolor)
                self.writeln( output[key] )
                self.writeln( """</tr>""")

            self.writeln("""
            <input type='hidden' name='_action_' value='edit'>
            <input type='hidden' name='edit_section' value='item'>
            <input type='hidden' name='serialNum' value='%s'>
            <tr><td height='40px' colspan=5 align=center><A HREF="./bom?serialNum=%s&new=1"><IMG SRC="../images/new.gif" onMouseDown="this.src='../images/newdown.gif';" onMouseOut="this.src='../images/new.gif';" style="border-style: none;"></A>&nbsp;<INPUT TYPE=image SRC="../images/submit1.gif" onMouseDown="this.src='../images/submit1down.gif';" onMouseOut="this.src='../images/submit1.gif';">&nbsp;<A HREF="./index"><IMAGE SRC="../images/back1.gif" onMouseDown="this.src='../images/back1down.gif';" onMouseOut="this.src='../images/back1.gif';" BORDER=0></A></td></tr>
            </form>
            </table>""" % (serialNum,serialNum) )

        self.writeln("""\t</td>\n</tr>\n</table>""")
