# vim: ts=4 et sts=4 sw=4 autoindent

from index import index
from lib.SecureSitePage import SecureSitePage
from etc import siteindex

from sndcs.CustomerFactory import CustomerFactory
from sndcs.ItemPriceByCustomerFactory import ItemPriceByCustomerFactory

class price_by_customer(index,SecureSitePage):
    
    def __init__(self):
        SecureSitePage.__init__(self, siteindex["dcs_edit_items"], 1, clearSearchResults = False)

    def localTitle(self):
        req = self.request()
        if req.hasField("new"):
            return "Create New Price By Customer"
        else:
            return "Item Prices by Customer"

    def writeJavascript(self):
        return """
        <script language='javascript' type='text/javascript'>
        function validOnSubmit(myForm) {
            if ( (myForm.CustomerNumber.value == "") ) {
                alert('Error: Please select a Customer');
                myForm.CustomerNumber.focus();
                return false;
            }

            if ( (myForm.CustomerPrice=="") ) {
                alert('Error: Please enter a valid Price');
                myForm.CustomerPrice.focus();
                return false;
            }
        return true;
        }
        </script>

        <script language='javascript' type='text/javascript'>
        function checkIfOpen(MyCheckbox,ItemPrice) {
            if (MyCheckbox.checked) {
                if (ItemPrice) {
                    ItemPrice.disabled=false;
                    return false;
                }
            }
            ItemPrice.disabled=true;
        }
        </script>
        """

    def newTab(self):
        req = self.request()
        serialNum = self.returnType(req.field("serialNum",""),int,None)
        if serialNum:
            return [ "New Price By Customer","price_by_customer?new=1&serialNum=%s" % serialNum  ]
        else:
            return index.newTab(self)

    def backPage(self):
        req = self.request()
        serialNum = self.returnType(req.field("serialNum",""),int,None)
        if serialNum:
            return "dcs_edit_items/price_by_customer?serialNum=%s" % (serialNum)
        else:
            return self.session().value("backPage",index.backPage(self) )

    def writeLocalContent(self):
        req = self.request()
        serialNum = self.returnType(req.field("serialNum",""),int,None)
        
        self.writeln("""<table width="100%">""")

        self.writeln( self.writeJavascript() )
        self.writeln( self.subTabs() )

        self.writeln("""<tr>\n\t<td>""")

        self.writeln("""<table align='center' width="50%">""")
        
        if not self.message == "":
            self.writeln( "<table align=center>%s</table>"  % self.message )

        elif req.hasField("new"):
            self.writeln("""
            <form action='./price_by_customer?_action_=edit&edit_type=customer_price' method='post' onSubmit='return validOnSubmit(this);'>
            <tr><th class=table_header align=center colspan=2>&nbsp;Create New Customer Price&nbsp;</th></tr>
            <tr><td class='body_desc'>&nbsp;Customer:&nbsp;</td><td align=right colspan=3><select style='width: 250px;' size=10 name='CustomerNumber'>""")

            for customerName in CustomerFactory().getCustomers():
                if not ItemPriceByCustomerFactory().getItemPriceByCustomerAndItemSerialNum(serialNum,customerName.serialNum() ):
                    self.writeln( self.write_option(customerName.serialNum(),"%s %s" % (customerName.customerId(),customerName.name() ) ) )
    
            self.writeln("""</select></td></tr>
            <tr><td colspan=2><hr/></td></tr>
            <tr><td class=body_desc>Customer Price:&nbsp;</td><td align=left>&nbsp;&nbsp;&nbsp;&nbsp;<input type='text' name='CustomerPrice' value='$0.00'></td></tr>
            <input type='hidden' name='serialNum' value='%s'>
            <tr height='50px'><td align=center colspan=2><INPUT TYPE=image SRC="../images/submit1.gif" onMouseDown="this.src='../images/submit1down.gif';" onMouseOut="this.src='../images/submit1.gif';"> &nbsp;<A HREF="./price_by_customer?serialNum=%s"><IMAGE SRC="../images/back1.gif" onMouseDown="this.src='../images/back1down.gif';" onMouseOut="this.src='../images/back1.gif';" BORDER=0></A> </td></tr>

            </form>
            </table>
            """ % ( serialNum,serialNum ) )

        else:
            self.writeln("""
            <form action='./price_by_customer' method='post'>
            <tr class=table_header><th align=left>&nbsp;Customer</th><th align=right>Item Price&nbsp;</th><th WIDTH='30px'>&nbsp;Edit&nbsp;</th><th WIDTH='30px'>&nbsp;Delete&nbsp;</th></tr>
            """)

            self.start_color()
            for customerPrice in ItemPriceByCustomerFactory().getItemPriceByItemSerialNum( int( serialNum ) ):
                    bgcolor = self.next_color()
                    self.writeln("""
                    <tr bgcolor='%s'><td class=list_data>%s - %s</td><td align=right><input disabled type='text' name='ItemPrice%s' size=7 value='$%.2f'></td>
                    <td align=center><input type='checkbox' name='editItemPrice%s' value=1 onClick='checkIfOpen(this,this.form.ItemPrice%s);'></td>
                    <td align=center><a href='./price_by_customer?_action_=delete&delete_type=itemPriceByCustomer&serialNum=%s'><img src='../images/button2.gif' border=0></a></td></tr>""" % (bgcolor,customerPrice.customer().customerId(),customerPrice.customer().name(),customerPrice.serialNum(),customerPrice.price(),customerPrice.serialNum(),customerPrice.serialNum(),customerPrice.serialNum()) )


            self.writeln("""
            <input type='hidden' name='_action_' value='edit'>
            <input type='hidden' name='edit_type' value='customer_price'>
            <tr><td colspan=4 align=center height='40px'><A HREF="./price_by_customer?new=1&serialNum=%s"><IMG SRC="../images/new.gif" onMouseDown="this.src='../images/newdown.gif';" onMouseOut="this.src='../images/new.gif';" style="border-style: none;"></A>&nbsp;<INPUT TYPE=image SRC="../images/submit1.gif" onMouseDown="this.src='../images/submit1down.gif';" onMouseOut="this.src='../images/submit1.gif';">&nbsp;<A HREF="./index"><IMAGE SRC="../images/back1.gif" onMouseDown="this.src='../images/back1down.gif';" onMouseOut="this.src='../images/back1.gif';" BORDER=0></A></td></tr>
            </form>
            </table>""" % serialNum)

        self.writeln("""\t</td>\n</tr>\n</table""")
