# vim: ts=4 et sts=4 sw=4 autoindent

from lib.SecureSitePage import SecureSitePage
from etc import siteindex
from index import index

from sndcs.UserFactory import UserFactory
from sndcs.ScrapCodeFactory import ScrapCodeFactory
from sndcs.UnitOfMeasureFactory import UnitOfMeasureFactory
from sndcs.ScrapCode import ScrapCode
from sndcs.Store import store

class scrapcodes(index,SecureSitePage):

        def __init__(self):
            SecureSitePage.__init__(self,siteindex["dcs_edit_items"],4)

        def localTitle(self):
            return "Scrap Codes"
    
        def backPage(self):
            return "dcs_edit_items/scrapcodes"

        def newTab(self):
            return [ "New Scrap Code", "edit_scrapcodes" ]

        def writeJavascript(self):
            return """
                <script language='javascript' type='text/javascript'>
                <!--
                function validOnSubmit(myForm) {
                    if ( myForm.Code.value == "" ) {
                        alert('Error: Enter a valid code.');
                        myForm.Code.focus();
                        return false;
                    }
                    
                    if ( myForm.Description.value == "" ) {
                        alert('Error: Enter a valid description.');
                        myForm.Description.focus();
                        return false;
                    }

                }
                //-->
                </script>"""


        def actions(self):
            return SecureSitePage.actions(self) + [ "edit", "delete", ]

        
        def edit(self):
            req = self.request()
            serialNum = self.returnType(req.field("serialNum",""),int,None)

            if serialNum:
                self.redirectIfTrue( UserFactory().redirectIfAccessDenied( self.session().value("userLoggedOn"), "edit_items" ) ) 
                tmpedit = ScrapCodeFactory().getScrapCodeBySerialNum(serialNum)
            else:
                self.redirectIfTrue( UserFactory().redirectIfAccessDenied( self.session().value("userLoggedOn"), "create_items" ) ) 
                tmpedit = ScrapCodeFactory().new()
            
            if store.fetchObjectsOfClass(ScrapCode,clauses = """WHERE code='%s' AND serialNum<>%s AND jobHedObjId IS NULL""" % (req.field("Code",""),tmpedit.serialNum() ) ):
                self.message = '<b>Error:</b> An ScrapCode already has that code.'
                self.message = """<table><tr align='center' width="100%%"><td><font color='#ff0000'>%s</font></td></tr></table>""" % self.message
                self.writeBody()
                return
            else:
                for scrap in store.fetchObjectsOfClass(ScrapCode,clauses = """WHERE code='%s' AND jobHedObjId IS NOT NULL""" % tmpedit.code() ):
                    scrap.setCode(req.field("Code"))
                    
                tmpedit.setCode(req.field("Code"))
                tmpedit.setDescription(req.field("Description","No Description"))
                store.addObject(tmpedit)
                
                if req.field("Active",""):
                    tmpedit.setActive(True)
                    tmpedit.add()
                else:
                    tmpedit.setActive(False)
                    tmpedit.delete()

            store.saveChanges()
            self.goToPage("dcs_edit_items/scrapcodes")

        def delete(self):
            req = self.request()
            serialNum = self.returnType(req.field("serialNum",""),int,None)

            self.redirectIfTrue( UserFactory().redirectIfAccessDenied( self.session().value("userLoggedOn"), "delete_items" ) ) 

            tmpdelete = ScrapCodeFactory().getScrapCodeBySerialNum(serialNum)
            if not store.fetchObjectsOfClass(ScrapCode,clauses = """WHERE code='%s' AND jobHedObjId IS NOT NULL""" % tmpdelete.code() ) and not tmpdelete.referencingObjectsAndAttrs():
                store.deleteObject(tmpdelete)
                tmpdelete.delete()
            else:
                tmpdelete.setActive(False)
                tmpdelete.delete() #Remove from clients

            store.saveChanges()

            self.goToPage(self.backPage())

        def writeLocalContent(self):
            self.writeln("""<table width="100%">""")

            self.writeln( self.subTabs() )
            self.writeln("""<tr align='center'>\n\t<td>""")
            
            if self.message:
                self.writeln( self.message )
            else:
                header = ("""<tr class=table_header><th width=10% align=left>&nbsp;Code</th><th align=left width=80%%>Description</th><th align=left>Active</th><th WIDTH='30px'>Edit&nbsp;</th><th WIDTH='30px'>&nbsp;Delete</th></tr>""")

                self.writeln("""<table width="100%">""")
                self.writeln("""<form action='./scrapcodes?_action_=edit' method='post'>""")
                self.writeln( header )

                self.start_color()
                for scrap in ScrapCodeFactory().getScrapCodes():
                        Active = self.ifTrueReturn(scrap.active(),"Yes","No")
                        bgcolor = self.next_color()
                        self.writeln("""<tr class=list_data bgcolor='%s'><td align="left">&nbsp;%s</td><td align=left>%s</td><td align=center>&nbsp;%s&nbsp;</td><td align=center><a href="./edit_scrapcodes?serialNum=%s"><img border=0 src='../images/button4.gif'></a></td><td align=center><a href="./scrapcodes?_action_=delete&serialNum=%s"><img border=0 src='../images/button3.gif'></a></td></tr>""" % ( bgcolor, scrap.code(),self.escape( scrap.description() ),Active,scrap.serialNum(),scrap.serialNum() ) )

                self.writeln( header )
                self.writeln("""
                <tr>
                    <td height='40px' colspan=6 align=center><A HREF="./edit_scrapcodes?new=1"><IMG SRC="../images/new.gif" onMouseDown="this.src='../images/newdown.gif';" onMouseOut="this.src='../images/new.gif';" style="border-style: none;"></A>
                 </td>
                </tr>
                </table>
                """)

            self.writeln("""\t</td>\n</tr>\n</table>""")
