# vim: ts=4 et sts=4 sw=4 autoindent

from index import index
from lib.SecureSitePage import SecureSitePage
from sndcs.JobHedFactory import JobHedFactory
from sndcs.CustomerFactory import CustomerFactory
from etc import siteindex

class search(index,SecureSitePage):

    def __init__(self):
        SecureSitePage.__init__(self,siteindex["dcs_edit_jobs"],2,False, SecureSitePage.mainScripts + [ "../js/jscalendar/calendar.js", "../js/jscalendar/calendar-en.js","../js/jscalendar/calendar-setup.js"])
        self.pageCount = 100

    def actions(self):
        return index.actions(self) + [ "search" ]

    def newTab(self):
        return [ "New Search", "search" ]

    def localTitle(self):
        return "Search Jobs"

    def backPage(self):
        return self.session().value("backPage","dcs_edit_jobs/search")

    def subTabs(self):
        return index.subTabs(self, add = self.write_div(id="subtab3",classname="subtab",text="New Job", construct = [ 3, siteindex["dcs_edit_jobs"], "./new_job" ]))

    def search(self):
        req = self.request()
        deleteValues = [ "x","y","_action_","page","jobHedDescription", "customerDescription" ]
        clauseString = ""
        
        oldTab = self.newTab
        def newTab(): return [ "New Job", "new_job" ]
        
        page = self.returnType(req.field("page",""),int,0) * self.pageCount

        if not req.field("customerObjId","") == "":
            clauseString += " customerObjId='%s' AND" % req.field("customerObjId")
            deleteValues.append("customerObjId")

        if not req.field("Complete","") == "":
            clauseString += " complete=%s AND" % self.escape( req.field("Complete") )
            deleteValues.append("Complete")

        if not req.field("jobNumber","") == "":
            clauseString += " number=%s AND" % self.escape( req.field("jobNumber") )
            deleteValues.append("jobNumber")

        if req.field("serialNum",""):
            self.__dict__.update({"newTab": newTab})
            clauseString += " serialNum=%s AND" % self.escape( req.field("serialNum") )
            deleteValues.append("serialNum")
   
        if req.field("clauseString",""):
            clauseString = req.field("clauseString")
            if clauseString.count("serialNum") > 0:
                self.__dict__.update({"newTab": newTab})
            deleteValues.append("clauseString")
       
        for key,value in dict(req.fields()).items():
            if not key in deleteValues and not value == "":
                clauseString += " %s LIKE '%%%s%%' AND" % (key,self.escape( value ))
        
        if clauseString: searchResult = JobHedFactory().searchJobHeds("WHERE %s" % clauseString[:-4])
        else: searchResult = JobHedFactory().getJobHeds()


        ### THIS MAY NEED TO BE DISABLED IN SOME ENVIRONMENTS BECAUSE OF RESOURCES ###
        def checkNote(item,section,data):
            if section == "jobHedDescription":
                notes = item.notes()
                if notes:
                    if notes[0].note():
                        if str(notes[0].note().body()).lower().count(data.lower()):
                            return True
            elif section == "customerDescription":
                if str(item.customer().customerId()).lower().count(data.lower()):
                    return True
            return False


        for y in [ "jobHedDescription", "customerDescription" ]:
            data = req.field(y,"")
            if data and searchResult:
                searchResult = [ x for x in searchResult if checkNote(x,y,data) ]

        message = ""
        message += ( self.writeJavascript() )
        
        message += ("""<table width="100%">""")
        message += ("""
                <form action='./search?_action_=edit&edit_section=job' method='post'>
                <tr class=refresh height=50px><td colspan=15 align=right><b><a href='./%s'>%s</a></b></td></tr>
                <tr class=table_header><th colspan=15>Search Results</th></tr>""" % ( list(self.newTab())[1], list(self.newTab())[0] ) )
        
        header = ("""<tr CLASS=table_header ALIGN=center>
                     <th ALIGN=left width=25%%>&nbsp;Job #</th>
                     <th width=15%% align=left>Customer #</th>
                     <th width=15%% align=left>&nbsp;Item #</th>
                     <th align=left>&nbsp;Open Date</th>
                     <th align=left>&nbsp;Due Date</th>
                     <th align=left width=10%%>&nbsp;Status</th>
                     <th>&nbsp;Reporting</th>
                     <th>&nbsp;Qty Req.</th>
                     <th>&nbsp;Qty Cmplt.</th>
                     <th>&nbsp;Qty. Scrap</th>
                     <th>Priority</th>
                     <th ALIGN=center>&nbsp;Hide&nbsp;</th>
                     <th ALIGN=center>&nbsp;Edit</th>
                     <th ALIGN=center>&nbsp;Notes</th>
                     <th ALIGN=center>&nbsp;Asm.</th></tr>""")
        message += ( header )

        if not searchResult:
            message += ("""<tr><td colspan=15 align=center><b>No Matching Records!</b></td></tr>""")
        else:
            self.start_color()
            numberOfResults = len(searchResult)
                
            for job in searchResult[page : (page + self.pageCount) ]:
                bgcolor = self.next_color()
                
                if job.customer(): customerfield = "%s" % (job.customer().name())
                else: customerfield = ""

                if job.item(): itemfield = "<b>%s:</b> %s" % (job.item().number(),job.item().description())
                else: itemfield = "DCS 1.0 Import"
            
                jobfield = "<b>%s</b> %s" % (job.number(),job.itemDescription() )
                    
                openDate = self.parseDate(job.openDate() )
                dueDate = self.parseDate(job.dueDate() )
                            
                message += ("""
                    <tr bgcolor='%s'>
                        <td class=list_data><nobr><b>%s</b>&nbsp;<input disabled type='text' name='ItemDescription%s' value=%s size=20 maxlength=100></nobr></td>
                        <td class=list_data><b>%s</b>&nbsp;&nbsp;</td>
                        <td class=list_data>%s&nbsp;&nbsp;</td>
                        <td class=list_data align=left>&nbsp;%s</td><td class=list_data align=left><pre>&nbsp;<input disabled type='text' id='DueDate%s' name='DueDate%s' size=10 maxlength=10 value='%s'><IMG ID='due_trigger%s' STYLE='visibility: hidden;' SRC=../images/button1.gif CLASS=pointer border=0></pre></td>
                        <td class=list_data><select disabled name='Complete%s'>""" % (bgcolor,job.number(),job.serialNum(),self.quoted_escape(job.itemDescription()),customerfield,itemfield,openDate,job.serialNum(),job.serialNum(),dueDate,job.serialNum(),job.serialNum() ) )

                for key,value in { "0": "Open","1": "Closed" }.items():
                    message += ( self.write_option(key,value, job.complete() == int(key) ) )

                jobHide = self.ifTrueReturn(job.hidden(),"CHECKED","")
                message += ("""\n\t</select>\n</td>
                        <td class=list_data align=center><a href='../dcs_admin/reports?_action_=reports&jobHedReport=1&jobHedSerialNum=%s'><img src='../images/button1.gif' border=0></a></td>
                        <td class=list_data align=center><input disabled type='text' name='QtyRequired%s' size=5 value='%s'></td>
                        <td class=list_data align=center><input disabled type='text' name='QtyCompleted%s' size=5 value='%s'></td>
                        <td class=list_data align=center><input disabled type='text' name='QtyScrap%s' size=5 value='%s'></td>
                        <td class=list_data align=center><input disabled type='text' size=4 maxlength=10 name='Priority%s' value='%s'></td>
                        <td class=list_data align=center><input disabled type='checkbox' name='Hidden%s' value=1 %s></td>
                        <td align=center><input type='checkbox' name='editJob%s' value=1 onClick='checkIfOpen(this,this.form.Priority%s,this.form.DueDate%s,this.form.QtyRequired%s,this.form.QtyCompleted%s,this.form.QtyScrap%s,this.form.Complete%s,this.form.Hidden%s,this.form.due_trigger%s,this.form.ItemDescription%s);'></td>
                        <td align=center><a href='./notes?serialNum=%s'><img src='../images/button3.gif' border=0></a></td>
                        <td align=center><a href='./assembly?serialNum=%s'><img src='../images/button2.gif' border=0></a></td>
                </tr>""" % (job.serialNum(),job.serialNum(),self.quoted_escape(job.qtyRequired()),job.serialNum(),self.quoted_escape(job.qtyComplete()),job.serialNum(),self.quoted_escape(job.qtyScrap()),job.serialNum(), self.quoted_escape(job.priority()), job.serialNum(),jobHide,job.serialNum(),job.serialNum(),job.serialNum(),job.serialNum(),job.serialNum(),job.serialNum(),job.serialNum(),job.serialNum(),job.serialNum(),job.serialNum(),job.serialNum(),job.serialNum() ) )

                    
                message+=("""
                    <script langauge="javascript">
                    Calendar.setup({
                            inputField     :    "DueDate%s",     // id of the input field
                            ifFormat       :    "%%m/%%d/%%Y",      // format of the input field
                            button         :    "due_trigger%s",  // trigger for the calendar (button ID)
                            align          :    "Bl",           // alignment (defaults to "Bl")
                            singleClick    :    true
                            });
                    </script>""" % (job.serialNum(),job.serialNum() ) )


            message += ( header )

            clauseString = self.convertToHexString(clauseString)
            message += ("""<tr class=refresh>\n\t<td colspan=8 align=left width="100%">&nbsp;""")
            if (page / self.pageCount) > 0:
                message += ("""<b><a href='./search?_action_=search&page=%s&x=&y=&clauseString=%s'>Prev %s Results</a></b>""" % (page / self.pageCount - 1 , clauseString, self.pageCount ) )

            message += ("""\t</td>\n\t<td colspan=7 align=right width="100%">""")
                
            if (numberOfResults > ( ( ( ( page + self.pageCount) / self.pageCount ) * self.pageCount)  + 1) ):
                nextpage = numberOfResults - page
                                
                if nextpage >= self.pageCount:
                    nextpage = 100
                                
                message += ("""<b><a href='./search?_action_=search&page=%s&x=&y=&clauseString=%s'>Next %s Results</a></b>""" % (page / self.pageCount + 1 , clauseString, nextpage ) ) 
                
        if searchResult:
            message += ("""\n\t</td>\n</tr>\n<tr><td align=center colspan=14 height='40px'><INPUT TYPE=image SRC="../images/submit1.gif" onMouseDown="this.src='../images/submit1down.gif';" onMouseOut="this.src='../images/submit1.gif';">""")

        if self.newTab()[0] == "New Job":
            message += ("""&nbsp;<A HREF="./new_job"><IMG SRC="../images/new.gif" onMouseDown="this.src='../images/newdown.gif';" onMouseOut="this.src='../images/new.gif';" style="border-style: none;"></A>""")

        message += ("""\n\t</td>\n</tr>\n</form>""")
        message += ("""</table>""")

        self.message = message
        self.writeBody()
        
        self.__dict__.update({"newTab": oldTab})

        self.session().setValue("backPage","dcs_edit_jobs/search?_action_=search&clauseString=%s&page=%s&x=&y=" % (clauseString,(page / self.pageCount) ) )
        self.setSearchResultsPage()

    def writeLocalContent(self):
        self.writeln("""<table width="100%">""")
        self.writeln( self.subTabs() )
        self.writeln("""<tr>\n\t<td>""")

        if self.message == "":
                self.writeln("""<table width="35%" align="center">""")
                self.writeln("""<form action='./search?_action_=search' method='post'>
                <tr class=table_header><th align=center colspan=4>Search Jobs</th></tr>
                <tr><td class=body_desc>&nbsp;Priority:&nbsp;</td><td align=left><input type=text" name="priority" size=8></td><td class=body_desc align=right>&nbsp;Job #&nbsp;</td><td align=left><input type="text" name="jobNumber" size=10></td></tr>
                <tr><td class=body_desc>&nbsp;Status:&nbsp;</td><td>
                        <select name="Complete">
                                <option value='' SELECTED>Any</option>
                                <option value='0'>Open</option>
                                <option value='1'>Closed</option>
                        </select>
                </td>
                <td class=body_desc align=right>&nbsp;Item #&nbsp;</td><td align=left><input type="text" name="itemNumber" size=10></td></tr>
                <tr><td class=body_desc align=left colspan=2>&nbsp;Customer Name:&nbsp;</td><td colspan=2 align=left><select name="customerObjId">\n<option value=''>Any</option>""")

                for customer in CustomerFactory().getCustomers():
                    self.writeln( self.write_option( customer.serialNum(),customer.name() ) )
                
                self.writeln("""\n\t</select>\n\t</td>\n</tr>
                <tr><td class=body_desc colspan=2>&nbsp;Description:&nbsp;</td><td align=center colspan=2><input type="text" name="itemDescription" size=30></td></tr>
                <tr><td class=body_desc colspan=2>&nbsp;JobHed Descriptions:&nbsp;</td><td align=center colspan=2><input type="text" name="jobHedDescription" size=30></td></tr>
                <tr><td class=body_desc colspan=2>&nbsp;Customer ID Search:&nbsp;</td><td align=center colspan=2><input type="text" name="customerDescription" size=30></td></tr>
                <tr><td align=center colspan=4 height='40px'><input type=image src="../images/search1.gif" onMouseDown="this.src='../images/search1down.gif';" onMouseOut="this.src='../images/search1.gif';">&nbsp;<A HREF="./index"><IMAGE SRC="../images/back1.gif" onMouseDown="this.src='../images/back1down.gif';" onMouseOut="this.src='../images/back1.gif';" BORDER=0></A></td></tr>                        
                </form>
                </table>""")
        else:
                self.writeln( self.message )

        self.writeln("""\t</td>\n</tr>\n</table>""")
