# vim: ts=4 et sts=4 sw=4 autoindent

from index import index
from lib.SecureSitePage import SecureSitePage
from etc import siteindex
from sndcs.ShiftFactory import ShiftFactory
from sndcs.UserFactory import UserFactory

from sndcs.Shift import Shift
from sndcs.Store import store

class shifts(index,SecureSitePage):

        def __init__(self):
                SecureSitePage.__init__(self,siteindex["dcs_edit_shifts"],0)

        def actions(self):
                return [ "edit", "delete" ]
        
        def writeJavascript(self):
                return """<script language='javascript' type='text/javascript'>
                <!--
                function validOnSubmit(myForm) {
                    if (myForm.Name.value == "") {
                        alert('Error: Enter a Description');
                        myForm.Name.focus();
                        return false;
                    }

                    if ( !isValidTime(myForm.StartTime.value) ) {
                        alert('Error: Enter a valid start time');
                        myForm.StartTime.focus();
                        return false;
                    }

                    if ( !isValidTime(myForm.EndTime.value) ) {
                        alert('Error: Enter a valid end time');
                        myForm.EndTime.focus();
                        return false;   
                    }
                    return true;
                }
                //-->
                </script>
                """
        
        def delete(self):
                req = self.request()
                serialNum = self.returnType(req.field("serialNum"),int,None)

                self.redirectIfTrue( UserFactory().redirectIfAccessDenied( self.session().value("userLoggedOn"), "delete_shifts" ) ) 
                tmpdelete = ShiftFactory().getShiftBySerialNum( serialNum )
                    
                if not tmpdelete.referencingObjectsAndAttrs():
                    store.deleteObject(tmpdelete)
                else:
                    tmpdelete.setActive(False)
                
                store.saveChanges()
                self.goToPage("dcs_edit_shifts")
                
        def edit(self):
                req = self.request()
                serialNum = self.returnType( req.field("serialNum",""),int, None)
                
                if serialNum:
                    self.redirectIfTrue( UserFactory().redirectIfAccessDenied( self.session().value("userLoggedOn"), "edit_shifts" ) ) 
                    if store.fetchObjectsOfClass(Shift,clauses = """WHERE serialNum<>'%s' AND name='%s'""" % (serialNum,req.field("Name","")) ):
                            self.message = ("""<b>Error:</b> An Shift already exists with that description""")
                            self.writeBody()
                            return

                    tmpedit = ShiftFactory().getShiftBySerialNum( serialNum )
                else:
                    self.redirectIfTrue( UserFactory().redirectIfAccessDenied( self.session().value("userLoggedOn"), "delete_shifts" ) ) 
                    
                    if store.fetchObjectsOfClass(Shift,clauses = """WHERE name='%s'""" % self.escape( req.field("Name","") ) ):
                            self.message = ("""<b>Error:</b> An Shift already exists with that description""")
                            self.writeBody()
                            return

                    tmpedit = ShiftFactory().new()
                
                startTime = self.convertToDateTime(req.field("StartTime","00:00:00"))
                endTime = self.convertToDateTime(req.field("EndTime","00:00:00"))
                description = req.field("Name","No Description")

                if not startTime:
                    self.message = ("""<b>Error:</b> Start Time is not valid. Please go back and enter it in the correct format.""")
                    self.writeBody()
                    return
                elif not endTime:
                    self.message = ("""<b>Error:</b> End Time is not valid. Please go back and enter it in the correct format.""")
                    self.writeBody()
                    return

                tmpedit.setActive( True == req.hasField("Status") )
                tmpedit.setStartTime(startTime)
                tmpedit.setEndTime(endTime)
                tmpedit.setName(description)

                store.addObject(tmpedit)
                store.saveChanges()

                self.goToPage("dcs_edit_shifts")
                
                
        def localTitle(self):
            if self.request().hasField("serialNum"):
                return "Edit Shift"
            else:
                return "Create New Shift"

        def writeLocalContent(self):
                req = self.request()
                serialNum = self.returnType( req.field("serialNum",""), int, None)

                if serialNum:
                        tmpedit = ShiftFactory().getShiftBySerialNum( serialNum )
                        greeting = "Edit"
                        name = tmpedit.name()
                        start = str(tmpedit.startTime() )[:8]
                        end = str(tmpedit.endTime() )[:8]
                        active = self.ifTrueReturn( tmpedit.active(), "CHECKED", "" )
                else: 
                        name = ""
                        greeting = "Create New"
                        active = "CHECKED"
                        start = "00:00:00"
                        end = "00:00:00"

                self.writeln( self.writeJavascript() )
            
                if not self.message == "":
                    self.writeln("""<table width="100%%"><tr align="center"><td><font color="#ff0000">%s</font></td></tr></table>""" % self.message )
                
                else:
                        self.writeln("""<table width="50%">""")
                        
                        self.writeln("""\n<form action="./shifts?_action_=edit" method="post" onSubmit='return validOnSubmit(this);'>
                        <tr CLASS=table_header><th COLSPAN=4>%s Shift </th></tr>
                        <tr>
                            <td ALIGN=right CLASS=body_desc>&nbsp;Description:&nbsp;</td>
                            <td ALIGN=left><INPUT TYPE=TEXT NAME=Name CLASS=body_data VALUE="%s"></td>
                        </tr>
                        <tr>
                            <td ALIGN=right CLASS=body_desc>&nbsp;Start Time (Military - HH:MM:SS):&nbsp;</td>
                            <td ALIGN=left><INPUT TYPE=TEXT NAME=StartTime CLASS=body_data SIZE=8 MAXLENGTH=8 VALUE="%s"></td>
                        </tr>
                        <tr>
                            <td ALIGN=right CLASS=body_desc>&nbsp;End Time (Military - HH:MM:SS):&nbsp;</td>
                            <td ALIGN=left><INPUT TYPE=TEXT NAME=EndTime CLASS=body_data SIZE=8 MAXLENGTH=8 VALUE="%s"></td>
                        </tr>
                        <tr>
                            <td ALIGN=right CLASS=body_desc>&nbsp;Active:&nbsp;</td>
                            <td ALIGN=left><INPUT TYPE=checkbox NAME=Status CLASS=body_data VALUE=1 %s>&nbsp;</td>
                        </tr>
                        <tr height='40px'><td colspan=2 align=center><INPUT TYPE=image SRC="../images/submit1.gif" onMouseDown="this.src='../images/submit1down.gif';" onMouseOut="this.src='../images/submit1.gif';">&nbsp;<A HREF="./index"><IMAGE SRC="../images/back1.gif" onMouseDown="this.src='../images/back1down.gif';" onMouseOut="this.src='../images/back1.gif';" BORDER=0></A></td></tr>""" % (greeting,name,start,end,active) );

                        if serialNum:
                            self.writeln("<input type='hidden' name='serialNum' value=%s>" % serialNum)
                        
                        self.writeln("</form>\n</table>")
