# vim: ts=4 et sts=4 sw=4 autoindent

from lib.SecureSitePage import SecureSitePage
from index import index
from etc import siteindex

from sndcs.UserFactory import UserFactory
from sndcs.NoteByLaborDtlFactory import NoteByLaborDtlFactory
from sndcs.OperationFactory import OperationFactory
from sndcs.AssemblyFactory import AssemblyFactory
from sndcs.JobHedFactory import JobHedFactory
from sndcs.IndirectFactory import IndirectFactory
from sndcs.LaborHedFactory import LaborHedFactory
from sndcs.LaborDtlFactory import LaborDtlFactory
from sndcs.Store import store
    
class labordtl(index,SecureSitePage):

        def __init__(self):
            SecureSitePage.__init__(self,0,0,False, SecureSitePage.mainScripts + [ "../js/jscalendar/calendar.js", "../js/jscalendar/calendar-en.js", "../js/jscalendar/calendar-setup.js" ])
                
        def localTitle(self):
            return "LaborDtl Records"
      
        def writeStyleSheet(self):
            SecureSitePage.writeStyleSheet(self)
            self.writeln("""<link rel="stylesheet" type="text/css" media="all" href="../css/calendar-blue.css" title="blue" />""")

        def actions(self):
            return SecureSitePage.actions(self) + [ "delete","edit","post_labor_dtl" ]

        def backPage(self):
            req = self.request()
            serialNum = self.returnType(req.field("serialNum",""),int,None)
            
            if serialNum:
                return "dcs_edit_time/labordtl?serialNum=%s" % serialNum
            else:
                return "dcs_edit_time"

        def delete(self):
                req = self.request()
                serialNum = self.returnType(req.field("serialNum",""),int,None)
                self.redirectIfTrue( UserFactory().redirectIfAccessDenied( self.session().value("userLoggedOn"), "delete_time" ) ) 
                
                tmpdelete = LaborDtlFactory().getLaborDtlBySerialNum( serialNum )
                store.deleteObject(tmpdelete)
                store.saveChanges()
                
                self.goToPage("dcs_edit_time/labordtl?serialNum=%s" % req.field("laborHedSerialNum",0) )

        def edit(self):
                req = self.request()
                serialNum = self.returnType(req.field("serialNum",""),int,None)
                
                if serialNum:
                        self.redirectIfTrue( UserFactory().redirectIfAccessDenied( self.session().value("userLoggedOn"), "create_time" ) ) 
                        
                        jobType = req.field("jobType","Indirect")
                        backPage = "dcs_edit_time/labordtl?serialNum=%s" % serialNum 
                        
                        if jobType == "Indirect":
                                startStamp = self.convertToDateTime(req.field("startStamp",False))
                                endStamp = self.convertToDateTime(req.field("endStamp",False))

                                laborHedObj = LaborHedFactory().getLaborHedBySerialNum( int( serialNum ) )# NOTE: The serialNum here is the LaborHed serial

                                tmpedit = LaborDtlFactory().new( indirect=IndirectFactory().getIndirectBySerialNum( int(req.field("Indirects") ) )  , jobType=jobType, startStamp=startStamp, startTerminal=self.terminalName ) 

                                if startStamp and endStamp:
                                    tmpedit.setStartStamp( startStamp )
                                    tmpedit.setEndStamp( endStamp )
                                else:
                                    store.saveChanges()
                                    self.goToPage(backPage)
                                
                                tmpedit.setLaborHed( laborHedObj )

                                store.addObject(tmpedit)
                                store.saveChanges()
                                
                                self.goToPage(backPage)
                                return

                        else: # Setup and Production jobs.
                                savefields = dict( req.fields() ) # Now see which assembly and operation the task is.
                                backPage = ""
                                if not savefields["jobHedNumber"]:
                                    jobList = self.returnType(savefields["JobItem"],[str,list],[])
                                else:
                                    jobList = self.returnType(savefields["jobHedNumber"],str,None)
                                    jobList = JobHedFactory().getJobHedByNumber(jobList)

                                    if jobList:
                                        jobList = [ jobList[0].serialNum() ]
                                    else:
                                        self.goToPage() 
                                        
                                if self.isType(jobList,int):
                                    jobList = [ savefields["JobItem"] ]
                                else:
                                    moreJobs = jobList[1:]
                                    for job in moreJobs:
                                        backPage += "&moreJob=%s" % job
                                
                                savefields["JobItem"] = jobList[0]
                                backPage = "dcs_edit_time/assembly?%s%s" % (self.ParseValues(savefields),backPage ) 
                                self.goToPage(backPage)

                elif req.hasField("laborDtlSerialNum"):
                        # MULTI-ENTRY Edit Below
                        self.redirectIfTrue( UserFactory().redirectIfAccessDenied( self.session().value("userLoggedOn"), "edit_time" ) ) 
                        for key,value in req.fields().items():
                                if key.count("editDtl"):
                                    serialNum = self.returnType( key.replace("editDtl",""), int, None)

                                    tmpedit = LaborDtlFactory().getLaborDtlBySerialNum(serialNum)
                                    laborHedSerial = tmpedit.laborHed().serialNum()

                                    startStamp = req.field("startStamp%s" % serialNum)
                                    endStamp = req.field("endStamp%s" % serialNum)

                                    if cmp(startStamp,endStamp) == 1:
                                        continue # startStamp can't logically be after endStamp
    
                                    startStamp = self.convertToDateTime(startStamp) 
                                    endStamp = self.convertToDateTime(endStamp) 
                                    
                                    if startStamp and endStamp:
                                        tmpedit.setStartStamp( startStamp )
                                        tmpedit.setEndStamp( endStamp )
                                    
                                    activityPercentage = self.returnType(req.field("activityPercentage%s" % serialNum, ""),int,100)

                                    if self.isType(activityPercentage,float):
                                        if activityPercentage > 100: activityPercentage = 100
                                        elif activityPercentage < 0: activityPercentage = 0
                                        if tmpedit.operation(): tmpedit.setActivityPercentage( float( activityPercentage ) )

                                    qtyComplete = self.returnType(req.field("qtyComplete%s" % serialNum,0),float,0) # Must be an indirect task
                                    qtyScrap = self.returnType(req.field("qtyScrap%s" % serialNum, 0),float,0)

                                    operation = tmpedit.operation()
                                    if operation:
                                        operation = operation.assembly().jobHed()
                                        
                                        # TODO: what do we do about the operation qty complete... do we subtract previous qty complete and add in new qty complete?
                                        #jobQtyComplete = int( operation.qtyComplete() ) + ( qtyComplete - tmpedit.qtyComplete() )
                                        #jobQtyScrap = int( operation.qtyScrap() ) + ( qtyScrap - tmpedit.qtyScrap() )

                                        tmpedit.setQtyComplete(qtyComplete)
                                        tmpedit.setQtyScrap(qtyScrap)
                                       
                                        if qtyComplete >= 0: operation.setQtyComplete(qtyComplete)
                                        if qtyScrap >= 0: operation.setQtyScrap(qtyScrap) 

                                        store.addObject(operation)
                                    tmpedit.setActiveTrans(0) #NOTE: Don't know if sndcs2_web or the wap that was causing activeTrans to be greater than 0.
                                    store.saveChanges() 
    
                        self.goToPage("dcs_edit_time/labordtl?serialNum=%s" % req.field("laborDtlSerialNum", 0) )

        def post_labor_dtl(self):
                    req = self.request()

                    qtyComplete = self.returnType( req.field("QtyComplete",0), float, 0 )
                    qtyScrap = self.returnType( req.field("QtyScrap",0), float, 0 )
                    activityPercentage = self.returnType( req.field("ActivityPercentage",100), float, 100 )
                    serialNum = self.returnType(req.field("serialNum",""),int, None)
        
                    self.redirectIfTrue( UserFactory().redirectIfAccessDenied( self.session().value("userLoggedOn"), "edit_time" ) ) 
                    laborHedObj = LaborHedFactory().getLaborHedBySerialNum( serialNum ) 

                    startStamp = self.convertToDateTime(req.field("startStamp",""))
                    endStamp = self.convertToDateTime(req.field("endStamp",""))

                    operation_serialNum = self.returnType(req.field("operation",""),int,0)
                    if operation_serialNum:
                        operation = OperationFactory().getOperationBySerialNum( operation_serialNum )
                        tmpedit = LaborDtlFactory().new(operation = operation,startStamp = startStamp,jobType = req.field("jobType","Production"), startTerminal = self.terminalName)
                        tmpedit.setEndStamp(endStamp)
                        tmpedit.setEndTerminal(self.terminalName)
                        
                        jobHedObj = operation.assembly().jobHed()
                        
                        jobComplete = self.returnType(jobHedObj.qtyComplete(),float,0)
                        jobScrap = self.returnType(jobHedObj.qtyScrap(),float,0)
                        
                        jobComplete += qtyComplete
                        jobScrap += qtyScrap
                        
                        tmpedit.setQtyComplete(jobComplete)
                        #jobHedObj.setQtyComplete(jobComplete)
                        
                        operation.setComplete( req.hasField("Complete") )
                        tmpedit.setQtyScrap(jobScrap)
                        #jobHedObj.setQtyScrap(jobScrap)

                        store.addObject(operation)
                        store.addObject(jobHedObj)
                        store.saveChanges()

                    tmpedit.setLaborHed( laborHedObj )
    
                    if tmpedit.operation():
                        tmpedit.setActivityPercentage( activityPercentage )

                    store.addObject(tmpedit)
                    store.saveChanges()
                    
                    if not req.field("Note","") == "":
                        comments = req.field("Note","")
                        tmpnew = NoteByLaborDtlFactory().new(comments)
                        tmpnew.setLaborDtl(tmpedit)

                        store.addObject(tmpnew)
                        store.saveChanges()

                    moreJob = self.returnType(req.field("moreJob",0),[int,list],[ False ])

                    if moreJob:
                        sendUrl = ""
                        savefields = dict( req.fields() )

                        if self.isType(moreJob,int):
                            jobItem = moreJob
                            savefields["moreJob"] = "False"
                        elif self.isType(moreJob,list) and not "".join(moreJob) == "False":
                            jobItem = savefields["moreJob"][0]    
                            moreJobs = savefields["moreJob"][1:]

                            for job in moreJobs:
                                sendUrl += "&moreJob=%s" % job
                        else:
                            self.goToPage("dcs_edit_time/labordtl?serialNum=%s" % serialNum)

                        deleteValues = ["QtyComplete", "QtyScrap","assembly","operation","Note","moreJob"]
                        savefields["JobItem"] = jobItem
                        savefields["PercentLeft"] = self.returnType(savefields["PercentLeft"],float,0.00) -  self.returnType(savefields["ActivityPercentage"],float,0.00)
                        sendUrl = "dcs_edit_time/assembly?%s%s" % (self.ParseValues(savefields,deleteValues = deleteValues),sendUrl)

                        self.goToPage(sendUrl)

        def newTab(self):
            return ["New LaborHed","edit_time"]


        def writeLocalContent(self):
                req=self.request()
                serialNum = self.returnType(req.field("serialNum",""),int,None)

                if self.message is "" and serialNum:
                    self.writeln( self.writeJavascript("checkIfOpen") )

                    self.writeln("""<table width="100%">""")
                    self.writeln("""
                    <form action='./labordtl?_action_=edit&laborDtlSerialNum=%s' method='post'>
                    <tr class=title align=center><td colspan=7>LaborDtl Records</td>
                    <td align=center colspan=3><a class=hyperlink href='javascript:window.close()'>Close Window</a></td></tr>""" % serialNum)

                    laborDtls = LaborDtlFactory().getLaborDtlsByLaborHedSerialNum(serialNum)
                    if laborDtls:

                        header = ("""
                        <tr class=table_header>
                            <th align=left width='10%'>Job Number&nbsp;</th>
                            <th align=left>&nbsp;Job Description</th><th>&nbsp;Assembly</th>
                            <th>&nbsp;Operation</th>
                            <th align=left>&nbsp;Start Stamp&nbsp;</th>
                            <th align=left>&nbsp;End Stamp&nbsp;</th>
                            <th align=left>&nbsp;Act. %&nbsp;</th>
                            <th>&nbsp;Qty Comp.&nbsp;</th>
                            <th>&nbsp;Qty Scrap&nbsp;</th>
                            <th>&nbsp;Edit</th>
                            <th>&nbsp;Scrap Codes</th>
                            <th>&nbsp;Notes</th>
                            <th>&nbsp;Delete</th>
                        </tr>
                        """)
                        self.writeln( header )


                    labor_details = {}
                    self.start_color()
                    for laborDtl in laborDtls:
                            if not laborDtl.posted():
                                bgcolor = self.next_color()
                                jobType = laborDtl.jobType()
                                if not jobType == "Indirect":
                                    jobHedObject = laborDtl.operation().assembly().jobHed()
                                    
                                    jobNumber = str(jobHedObject.number() ).replace(" ","")
                                    job = str(jobHedObject.itemDescription() )
                                    if jobHedObject.customer():
                                        job = "%s - %s" % (jobHedObject.customer().name(), job)

                                    assemblyObj = laborDtl.operation().assembly()
                                    assembly = "%s %s" % (assemblyObj.number(),assemblyObj.description())

                                    operationObj = laborDtl.operation()
                                    operation = "%s %s" % (operationObj.number(),operationObj.description())
                                else:
                                    assembly,operation = [ "N/A","N/A" ]
                                    jobHedObject = laborDtl.indirect()
                                    jobNumber = jobHedObject.code()
                                    job = jobHedObject.description()

                                startDate = self.parseDate(str( laborDtl.startStamp() ) [:10])
                                startTime = str(laborDtl.startStamp())[11:19]
                                    
                                endDate = self.parseDate(str( laborDtl.endStamp() ) [:10])
                                endTime = str(laborDtl.endStamp())[11:19]

                                laborDtlNote = NoteByLaborDtlFactory().findNote(laborDtl.serialNum() )
                                if laborDtlNote: laborDtlNote = "Note: " + laborDtlNote[0].note().body()
                                else: laborDtlNote = ""

                                if laborDtl.operation():
                                    ActivityPercent = self.escape(laborDtl.activityPercentage())
                                    QtyComp = self.escape(laborDtl.qtyComplete(),"0.0")
                                    QtyScrap = self.escape(laborDtl.qtyScrap(),"0.0")
                                else:
                                    ActivityPercent = "N/A"
                                    QtyComp = "N/A"
                                    QtyScrap = "N/A"
                                
                                self.writeln("""
                                <tr class=%s bgcolor=%s title="%s"><td class=list_data><nobr>%s&nbsp;&nbsp;</nobr></td>
                                <td class=list_data><nobr>%s&nbsp;&nbsp</nobr></td>
                                <td class=list_data><nobr>%s&nbsp;&nbsp;</nobr></td>
                                <td class=list_data><nobr>%s&nbsp;&nbsp;</nobr></td>
                                <td class=list_data align=center><pre><input disabled type='text' id='startStamp%s' name='startStamp%s' value='%s' size=19 maxlength=19><IMG ID=start_trigger%s STYLE='visibility: hidden;' SRC=../images/button1.gif CLASS=pointer border=0></pre></td>
                                <td class=list_data align=center><pre><input disabled type='text' id='endStamp%s' name='endStamp%s' value='%s' size=19 maxlength=19><IMG ID=end_trigger%s STYLE='visibility: hidden;' SRC=../images/button1.gif CLASS=pointer border=0></pre></td>
                                <td class=list_data align=center><input disabled type='text' name='activityPercentage%s' value='%s' size=5></td>
                                <td class=list_data align=center><input disabled type='text' name='qtyComplete%s' value='%s' size=5></td>
                                <td class=list_data align=center><input disabled type='text' name='qtyScrap%s' value='%s' size=5></td>
                                
                                <td align=center><input type='checkbox' name='editDtl%s' value='1' onClick='checkIfOpen(this,this.form.startStamp%s,this.form.endStamp%s,this.form.activityPercentage%s,this.form.qtyComplete%s,this.form.qtyScrap%s,this.form.start_trigger%s,this.form.end_trigger%s,this.form.itemDescription);'></td>
                                <td align=center><a href="./scrapcodes?laborDtlSerialNum=%s&jobType=%s"><img src="../images/button1.gif" border=0></a></td>
                                <td align=center><a href="./notes?serialNum=%s&jobType=%s"><img src="../images/button2.gif" border=0></a></td>
                                <td align=center><a href='./labordtl?_action_=delete&serialNum=%s&laborHedSerialNum=%s'><img src='../images/button3.gif' border=0></a></td></tr>""" % (jobType.lower(),bgcolor,laborDtlNote,jobNumber,job,assembly,operation,laborDtl.serialNum(),laborDtl.serialNum(),("%s %s" % (startDate,startTime) ),laborDtl.serialNum(),laborDtl.serialNum(),laborDtl.serialNum(),("%s %s" % (endDate, endTime ) ),laborDtl.serialNum(),laborDtl.serialNum(),ActivityPercent,laborDtl.serialNum(),QtyComp,laborDtl.serialNum(),QtyScrap,laborDtl.serialNum(),laborDtl.serialNum(),laborDtl.serialNum(),laborDtl.serialNum(),laborDtl.serialNum(),laborDtl.serialNum(),laborDtl.serialNum(),laborDtl.serialNum(),laborDtl.serialNum(),laborDtl.jobType(),laborDtl.serialNum(),laborDtl.jobType(),laborDtl.serialNum(),laborDtl.laborHed().serialNum() ) )

                                self.writeln( self.writeJavascript(serialNum=laborDtl.serialNum(),section="dateDisplay") )

                    if laborDtls:
                        self.writeln( header )
                    
                    self.writeln("""
                    <tr height='40px'><td colspan=14 align=center><A HREF="./edit_labordtl?serialNum=%s"><IMG SRC="../images/new.gif" onMouseDown="this.src='../images/newdown.gif';" onMouseOut="this.src='../images/new.gif';" style="border-style: none;"></A>""" % serialNum )

                    if laborDtls:
                        self.writeln("""&nbsp;<INPUT TYPE=image SRC="../images/submit1.gif" onMouseDown="this.src='../images/submit1down.gif';" onMouseOut="this.src='../images/submit1.gif';">""")
       
                    self.writeln("""\n\t</td>\n</tr>\n</form>\n</table>""")
                else:
                    self.writeln("""<table width=100%%><tr align=center><td><font color='#ff0000'>%s</a></td></tr></table""" % self.message)
