# vim: ts=4 et sts=4 sw=4 autoindent

from lib.SecureSitePage import SecureSitePage
from etc import siteindex
from sndcs.EmployeeFactory import EmployeeFactory
from sndcs.IndirectFactory import IndirectFactory
from sndcs.DepartmentFactory import DepartmentFactory
from sndcs.UserFactory import UserFactory
from sndcs_common.PyroProxyMixin import PyroProxyMixin
from sndcs.Config import config
from index import index


class globalindirect(index,SecureSitePage, PyroProxyMixin):
    def __init__(self):
        SecureSitePage.__init__(self,siteindex["dcs_employee_list"],2,True)
        self.namespace = config.get("pyro", "namespace", "sndcs")

    def actions(self):
        return SecureSitePage.actions(self) + [ "indirectglobal" ]

    def indirectglobal(self):
        import Pyro.core
        employeeObject = Pyro.core.getProxyForURI(self.formatPyronameString(self.namespace, ["Employee"]))

        req = self.request()
        self.redirectIfTrue( UserFactory().redirectIfAccessDenied( self.session().value("userLoggedOn"), "admin_globals" ) ) 

        department = req.field("department",False)
        indirect = req.field("indirect",False)
        
        if department and indirect:
                for department_employee in EmployeeFactory().getEmployeesOfDepartmentCode( department ):
                    employee = department_employee.employee()
                    if employee.activeLaborHed() and not employee.isSuspended() and not bool([ x.indirect().serialNum() for x in employee.getActiveLaborDetails() if x.indirect() and int(x.indirect().serialNum())==int(indirect) ]): # Only can be logged on one global indirect at a time
                         
                        employee_proxy = employeeObject.setEmployeeBySerialNum( employee.serialNum() )
                        employee_proxy.startIndirect(indirect_serial_num = int(indirect), terminal_id = self.terminalName, global_indirect = True )

        self.goToPage("dcs_employee_list")

    def writeLocalContent(self):
        departmentList, indirectList = [ "","" ]

        for department in DepartmentFactory().getActiveDepartments():
                departmentList += ( self.write_option( department.code(), "%s - %s" % (department.code(),department.description() ) ) )
        
        for indirect in IndirectFactory().getAllActiveIndirects():
                indirectList += ( self.write_option( indirect.serialNum(), "%s - %s" % (indirect.code(),indirect.description() ) ) )

        self.writeln("""<table width="100%">""")
        self.writeln( self.subTabs() )

        self.writeln("""<tr>\n\t<td>""")
        self.writeln("""<table width="50%" align=center>""")
        self.writeln("""<form action='./globalindirect?_action_=indirectglobal' method='post'>""")
        self.writeln("""<tr class=table_header height='12px'><th colspan=2></th></tr>""")

        self.writeln("""<tr>\n\t<td class=body_desc align=right width="50%%">Select Department:&nbsp;</td>\n\t<td align=left><select name='department'>%s</select></td>\n</tr>""" % departmentList)
        self.writeln("""<tr>\n\t<td class=body_desc align=right width="50%%">Select Indirect Activity:&nbsp;</td>\n\t<td align=left><select name='indirect'>%s</select></td>\n</tr>""" % indirectList)
        self.writeln("""<tr>\n\t<td colspan=2 align=center height='40px'><INPUT TYPE=image SRC="../images/submit1.gif" onMouseDown="this.src='../images/submit1down.gif';" onMouseOut="this.src='../images/submit1.gif';" >&nbsp;<A HREF="./index"><IMAGE SRC="../images/back1.gif" onMouseDown="this.src='../images/back1down.gif';" onMouseOut="this.src='../images/back1.gif';" BORDER=0></A>\n\t</td>\n</tr>""")
        
        self.writeln("""</form>\n</table>\n\t</td>\n</tr>\n</table>""")
