# vim: ts=4 et sts=4 sw=4 autoindent

from lib.WapSecureSitePage import SecureSitePage
from sndcs.LaborDtlFactory import LaborDtlFactory
from sndcs.NoteByLaborDtlFactory import NoteByLaborDtlFactory
from sndcs.Store import store
from sndcs_common.PyroProxyMixin import PyroProxyMixin
import Pyro.core

class view(SecureSitePage, PyroProxyMixin):
    def __init__(self):
        SecureSitePage.__init__(self, 0)

    def localTitle(self):
        return "View Job"

    def actions(self):
        return SecureSitePage.actions(self) + ["endJob"]

    def endJob(self):
        req = self.request()
        if req.hasField("serialNum"):
            serialNum = req.field("serialNum")
            notes = req.field("notes","")

        self.writeln("%s" % notes)

        employee_factory = Pyro.core.getProxyForURI(self.formatPyronameString(self.namespace, ["Employee"]))
        employee = employee_factory.setEmployeeBySerialNum(int(serialNum))
        employeeLaborDtl = employee.getActiveLaborDetails()

        if notes:
            addNote = NoteByLaborDtlFactory().new(notes)
            addNote.setLaborDtl(LaborDtlFactory().getLaborDtlBySerialNum( employeeLaborDtl[0]["labordtl_id"]) )
            store.addObject(addNote)
            store.saveChanges()

        employee.endAllLaborDtl(terminal_id = self.terminal_name)
        employee.disconnect()
        self.goToPage("./dcs_current_activity/index?serialNum=" + serialNum)
        
        
    def writeLocalContent(self):
        req = self.request()
        if req.hasField("serialNum"):
            serialNum = req.field("serialNum")

        employee_factory = Pyro.core.getProxyForURI(self.formatPyronameString(self.namespace, ["Employee"]))
        employee = employee_factory.setEmployeeBySerialNum(int(serialNum))

        self.writeln("<table width=100%%>")
        self.writeln("<tr class=list_data><td>&nbsp;</td><td align=right><a href='index?serialNum=%s'>back</a></td></tr>" % serialNum)
        self.writeln("<tr class=list_data><td>Name:</td><td>%s</td></tr>" % employee.properName())
        
        activities = employee.getActivities()
        for act in activities:
            self.writeln("<tr class=list_data><td>Job Type:</td><td>%s</td></tr>" % act['type'])
            self.writeln("<tr class=list_data><td colspan=2>Job Description:</td></tr>")
            self.writeln("<tr class=list_data><td colspan=2><b>%s</b></td></tr>" % act['description'])


        self.writeln("""<tr class=list_data>
        <td colspan=2 align=center>
        <form method=post action=view>
        <input type=hidden name=serialNum value="%s">
        <input type=hidden name="_action_" value="endJob">""" % serialNum )
        
        if activities and not activities[0]['type'] == "indirect":
            self.writeln("""Notes<br>\n<input type="text" name="notes"><br>""")
        
        self.writeln("""<input type="submit" class="submit" value="End Job">\n</form></td></tr>""")
        
        self.writeln("</table>")
        employee.disconnect()
